package bomb;

import java.awt.Dimension;
import java.awt.Graphics2D;
import java.util.Vector;

public class BombEditor implements BombInterface
{
	/** The time at which the last rendering looped started from the point of view of the game logic */
	private long lastLoopTime = BombSystemTimer.getTime();
	
	private BombMain 			bombMain;

	private BombEditorGround	bombEditorGround;
	private BombEditorHud		bombEditorHud;
	
	private int					loadLevel, temp;
	
	private boolean				bLogic;	
	
	public BombEditor( BombMain bombMain, BombImage bombImage )
	{
		this.bombMain			= bombMain;
		
		this.bombEditorGround	= new BombEditorGround( this, bombImage );
		this.bombEditorHud		= new BombEditorHud( this, bombImage );
		
		this.loadLevel			= -1;
		this.temp				= -1;
	}

	public boolean getRenderAble()
	{
		return true;
	}

	public boolean getLogicAble()
	{
		return this.bLogic;
	}

	public void frameRenderer(Graphics2D g)
	{
		BombSystemTimer.sleep(lastLoopTime+20-BombSystemTimer.getTime());
		
		// work out how long its been since the last update
		lastLoopTime = BombSystemTimer.getTime();
		
		this.logic();
		
		this.bombEditorGround.render( g );
		this.bombEditorHud.render( g );
		
	}

	public void logic()
	{
		if ( this.loadLevel != -1 )
		{
			if ( this.loadLevel == 0 )
			{
				if ( this.temp != -1 )
				{
					this.bombMain.load( true );
					this.bombEditorHud.setText( this.bombMain.getBombLevelEditor() );
				} else
					this.temp	= 0;
				BombLevel bL	= this.bombMain.getBombLevelEditor();
				this.setVectorFinish( bL.getFinishVector( this.temp ) );
				this.setVectorSweety( bL.getSweetyVector( this.temp ) );
				this.setVectorWall( bL.getWallVector( this.temp ) );
				this.setVectorBrick( bL.getBrickVector( this.temp ) );
				this.bombEditorHud.setAllVector();
			}
			this.loadLevel	= -1;
			this.temp		= -1;
		}
	}

	public Dimension getPreferredSize()
	{
		return new Dimension( 640, 480);
	}
	
	public void setMenu()
	{
		this.bombMain.setMenu();
	}
	
	public void setNewSweety()
	{
		this.bombEditorGround.setNewSweety();
		this.bombEditorGround.setbFinish( false );
	}
	
	public Vector<BombSweety> getvSweety()
	{
		return this.bombEditorGround.getvSweety();
	}
	
	public void setNewWall()
	{
		this.bombEditorGround.setNewWall();
		this.bombEditorGround.setbFinish( false );
	}
	
	public Vector<BombWall> getvWall()
	{
		return this.bombEditorGround.getvWall();
	}
	
	public void setNewFinish( boolean bFinish )
	{
		this.bombEditorGround.setbFinish( bFinish );
	}
	
	public Vector<BombFinish> getvFinish()
	{
		return this.bombEditorGround.getvFinish();
	}
	
	public void setNewBrick()
	{
		this.bombEditorGround.setNewBrick();
		this.bombEditorGround.setbFinish( false );
	}
	
	public Vector<BombBrick> getvBrick()
	{
		return this.bombEditorGround.getvBrick();
	}
	
	public void setSelected( int selected )
	{
		this.bombEditorHud.setSelected( selected );
	}
	
	public void setDeleteSelected( int selected )
	{
		this.bombEditorHud.setDeleteSelected( selected );
	}
	
	public void setVectorWall( Vector<BombWall> vWall )
	{
		this.bombEditorGround.setvWall( vWall );
	}
	
	public void setVectorBrick( Vector<BombBrick> vBrick )
	{
		this.bombEditorGround.setvBrick( vBrick );
	}
	
	public void setVectorSweety( Vector<BombSweety> vSweety )
	{
		this.bombEditorGround.setvSweety( vSweety );
	}
	
	public void setVectorFinish( Vector<BombFinish> vFinish )
	{
		this.bombEditorGround.setvFinish( vFinish );
	}
	
	public void save( Vector<BombWall> vWall, Vector<BombBrick> vBrick, Vector<BombSweety> vSweety, Vector<BombFinish> vFinish, int bombCount, String name, int level )
	{
		this.bombMain.save( vWall, vBrick, vSweety, vFinish, bombCount, name, level );
	}
	
	public void load( int i )
	{
		this.loadLevel	= 0;
		this.temp		= i;
	}
	
	public void loadChoose( Vector<BombWall> vWall, Vector<BombBrick> vBrick, Vector<BombSweety> vSweety, Vector<BombFinish> vFinish )
	{
		this.setVectorFinish( vFinish );
		this.setVectorSweety( vSweety );
		this.setVectorWall( vWall );
		this.setVectorBrick( vBrick );
	}
	
	public BombLevel getBombLevel()
	{
		return this.bombMain.getBombLevelEditor();
	}
	
	public void setBombLevel( BombLevel bL )
	{
		this.bombMain.setBombLevel( bL );
	}
	
	public void setBombLevelHud( BombLevel bL )
	{
		this.bombEditorHud.setBombLevel( bL );
	}
	
	public void mousePressed( int x, int y )
	{
		if ( y < 400 )
			this.bombEditorGround.mousePressed( x, y );
		else
			this.bombEditorHud.mousePressed( x, y );
	}
	
	public void mouseReleased( int x, int y )
	{
		if ( y < 400 )
			this.bombEditorGround.mouseReleased( x, y );
		else
			this.bombEditorHud.mouseReleased( x, y );
	}
	
	public void mouseReleasedRight( int x, int y )
	{
		if ( y < 400 )
			this.bombEditorGround.mouseReleasedRight( x, y );
	}
	
	public void mouseMoved( int x, int y )
	{
		if ( y < 400 )
			this.bombEditorGround.mouseMoved( x, y );
		else
			this.bombEditorHud.mouseMoved( x, y );
	}
	
	public void mouseDragged( int x, int y )
	{
		if ( y < 400 )
			this.bombEditorGround.mouseDragged( x, y );
	}
	
	public void keyReleased( int key, char character )
	{
		this.bombEditorHud.keyReleased( key, character );
	}
	
}
