package bomb;

import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.image.BufferedImage;
import java.util.Vector;

import javax.swing.JPanel;

public class BombEditorHud extends JPanel
{
	private static final long serialVersionUID = 1L;

	private BombImage				bombImage;
	
	private BombEditor				bombEditor;
	
	private BufferedImage			iBackground, iMiddle, iWall, iBrick,
									iSweety, iFinish,
									iMiddleBomb, iMiddleChoose, iMiddleChooseChoose;
	
	private BombLevel				bombLevel;
	
	private BombButton[]			Buttons;

	private boolean 				bHud;
	
	private int 					selected, level;
	
	private Vector<BombSweety>		vSweety;
	private Vector<BombWall>		vWall;
	private Vector<BombFinish>		vFinish;
	private Vector<BombBrick>		vBrick;
	
	private BombPanel				panel;
	
	public BombEditorHud(BombEditor bombEditor, BombImage bombImage )
	{
		
		this.bombEditor		= bombEditor;
		this.bombImage		= bombImage;

		this.panel			= new BombPanel( this.bombImage );
		
		this.iBackground			= this.bombImage.setPicsMain( "/images/game_background_hud.png", false );
		this.iMiddle				= this.bombImage.setPicsMain( "/images/game_hud_editor_middle.png", false );
		this.iWall					= this.bombImage.setPicsMain( "/images/Wall.png", false );
		this.iSweety				= this.bombImage.setPicsMain( "/images/Sweety_Normal.png", false );
		this.iFinish				= this.bombImage.setPicsMain( "/images/Ziel.png", false );
		this.iMiddleBomb			= this.bombImage.setPicsMain( "/images/game_hud_editor_middlebomb.png", false );
		this.iMiddleChoose			= this.bombImage.setPicsMain( "/images/game_hud_editor_choose.png", false);
		this.iMiddleChooseChoose	= this.bombImage.setPicsMain( "/images/game_hud_editor_choose_choose.png", false);
		this.iBrick					= this.bombImage.setPicsMain( "/images/Brick.png", false );
		
		this.Buttons			= new BombButton[17];
		this.Buttons[0]			= new BombButton( this.bombImage.setPicsMain( "/images/game_hud_button_menu.png", false ),        585, 410, 50, 60, "Menu" );
		this.Buttons[1]			= new BombButton( this.bombImage.setPicsMain( "/images/game_hud_button_save.png", false ),        533, 412, 50, 25, "Save" );
		this.Buttons[2]			= new BombButton( this.bombImage.setPicsMain( "/images/game_hud_button_load.png", false ),        533, 443, 50, 25, "Load" );
		this.Buttons[3]			= new BombButton( this.bombImage.setPicsMain( "/images/game_hud_button_wall.png", false ),          5, 405, 30, 30, "Wall" );
		this.Buttons[4]			= new BombButton( this.bombImage.setPicsMain( "/images/game_hud_button_brick.png", false ),         5, 445, 30, 30, "Brick" );
		this.Buttons[5]			= new BombButton( this.bombImage.setPicsMain( "/images/game_hud_button_sweety.png", false ),       40, 405, 30, 30, "Sweety" );
		this.Buttons[6]			= new BombButton( this.bombImage.setPicsMain( "/images/game_hud_button_finish.png", false ),       40, 445, 30, 30, "Finish" );
		this.Buttons[7]			= new BombButton( this.bombImage.setPicsMain( "/images/game_hud_button_ok.png", false ),           85, 406, 40, 22, "Ok" );
		this.Buttons[8]			= new BombButton( this.bombImage.setPicsMain( "/images/game_hud_button_delete.png", false ),      235, 406, 40, 22, "Delete" );
		this.Buttons[9]			= new BombButton( this.bombImage.setPicsMain( "/images/game_hud_button_left.png", false ),        130, 407, 20, 20, "Left" );
		this.Buttons[10]		= new BombButton( this.bombImage.setPicsMain( "/images/game_hud_button_right.png", false ),       210, 407, 20, 20, "Right" );
		this.Buttons[11]		= new BombButton( this.bombImage.setPicsMain( "/images/game_hud_button_up.png", false ),          490, 405, 20, 20, "UpChoose" );
		this.Buttons[12]		= new BombButton( this.bombImage.setPicsMain( "/images/game_hud_button_up_red.png", false ),      510, 405, 20, 20, "UpChange" );
		this.Buttons[13]		= new BombButton( this.bombImage.setPicsMain( "/images/game_hud_button_down.png", false ),        490, 455, 20, 20, "DownChoose" );
		this.Buttons[14]		= new BombButton( this.bombImage.setPicsMain( "/images/game_hud_button_down_red.png", false ),    510, 455, 20, 20, "DownChange" );
		this.Buttons[15]		= new BombButton( this.bombImage.setPicsMain( "/images/game_hud_button_newLevel.png", false ),    490, 430, 20, 20, "NewLevel" );
		this.Buttons[16]		= new BombButton( this.bombImage.setPicsMain( "/images/game_hud_button_deleteLevel.png", false ), 510, 430, 20, 20, "DeleteLevel" );
		
		this.bHud				= false;
		
		this.vFinish			= new Vector<BombFinish>();
		this.vSweety			= new Vector<BombSweety>();
		this.vWall				= new Vector<BombWall>();
		this.vBrick				= new Vector<BombBrick>();
		
		this.selected			= -1;
		this.level				= 0;
	}

	public void setBombLevel( BombLevel bombLevel )
	{
		this.bombLevel		= bombLevel;
		this.setAllVector();
	}
	
	public void setAllVector()
	{
		this.vFinish	= this.bombEditor.getvFinish();
		this.vSweety	= this.bombEditor.getvSweety();
		this.vWall		= this.bombEditor.getvWall();
		this.vBrick		= this.bombEditor.getvBrick();
	}
	
	public void setbHud( boolean bHud )
	{
		this.bHud	= bHud;
		if ( this.bHud )
		{
			this.panel.textX.setVisible( true );
			this.panel.textY.setVisible( true );
			this.panel.textWidth.setVisible( this.bHud );
			this.panel.textHeight.setVisible( this.bHud );
		} else
		{
			this.panel.textX.setVisible( true );
			this.panel.textY.setVisible( true );
			this.panel.textWidth.setVisible( this.bHud );
			this.panel.textHeight.setVisible( this.bHud );
		}
	}
	
	public void setSelected( int selected )
	{
		if ( selected != this.selected )
		{
			if ( this.selected <= 0 )
			{}
			else if ( this.selected < 1000 )
			{
				(this.vWall.get( this.selected-1 )).setbSelected( false );
				this.bombEditor.setVectorWall( this.vWall );
			} else if ( this.selected < 2000 )
			{
				(this.vSweety.get( this.selected-1-1000 )).setbSelected( false );
				this.bombEditor.setVectorSweety( this.vSweety );
			} else if ( this.selected < 3000 )
			{
				(this.vFinish.get( this.selected-1-2000 )).setbSelected( false );
				this.bombEditor.setVectorFinish( this.vFinish );
			} else
			{
				(this.vBrick.get( this.selected-1-3000 )).setbSelected( false );
				this.bombEditor.setVectorBrick( this.vBrick );
			}
		}
		this.selected	= selected;
		if ( this.selected < 0 )
		{
			this.setbHud( false );
			this.panel.textX.setText( "" );
			this.panel.textY.setText( "" );
			this.panel.textX.setVisible( false );
			this.panel.textY.setVisible( false );
			return;
		}
		if ( this.selected < 1000 )
		{
			this.setbHud( false );
			this.panel.textX.setText( ""+(this.vWall.get( this.selected-1 )).getX() );
			this.panel.textY.setText( ""+(this.vWall.get( this.selected-1 )).getY() );
			(this.vWall.get( this.selected-1 )).setbSelected( true );
			this.bombEditor.setVectorWall( this.vWall );
		} else if ( this.selected < 2000 )
		{
			this.setbHud( false );
			this.panel.textX.setText( ""+(int)(this.vSweety.get( this.selected-1-1000 )).getX() );
			this.panel.textY.setText( ""+(int)(this.vSweety.get( this.selected-1-1000 )).getY() );
			(this.vSweety.get( this.selected-1-1000 )).setbSelected( true );
			this.bombEditor.setVectorSweety( this.vSweety );
		} else if ( this.selected < 3000 )
		{
			this.setbHud( true );
			this.panel.textX.setText( ""+(this.vFinish.get( this.selected-1-2000 )).getX() );
			this.panel.textY.setText( ""+(this.vFinish.get( this.selected-1-2000 )).getY() );
			this.panel.textHeight.setText( ""+(this.vFinish.get( this.selected-1-2000 )).getHeight() );
			this.panel.textWidth.setText( ""+(this.vFinish.get( this.selected-1-2000 )).getWidth() );
			(this.vFinish.get( this.selected-1-2000 )).setbSelected( true );
			this.bombEditor.setVectorFinish( this.vFinish );
		} else
		{
			this.setbHud( false );
			this.panel.textX.setText( ""+(this.vBrick.get( this.selected-1-3000 )).getX() );
			this.panel.textY.setText( ""+(this.vBrick.get( this.selected-1-3000 )).getY() );
			(this.vBrick.get( this.selected-1-3000 )).setbSelected( true );
			this.bombEditor.setVectorBrick( this.vBrick );
		}
	}
	
	public void setDeleteSelected( int minusSelected )
	{
		if ( ( minusSelected == -1 ) && ( this.selected > 0 ) )
		{
			if ( this.selected < 1000 )
			{
				(this.vWall.get( this.selected-1 )).setbSelected( false );
				this.bombEditor.setVectorWall( this.vWall );
			} else if ( this.selected < 2000 )
			{
				(this.vSweety.get( this.selected-1-1000 )).setbSelected( false );
				this.bombEditor.setVectorSweety( this.vSweety );
			} else if ( this.selected < 3000 )
			{
				(this.vFinish.get( this.selected-1-2000 )).setbSelected( false );
				this.bombEditor.setVectorFinish( this.vFinish );
			} else
			{
				(this.vBrick.get( this.selected-1-3000 )).setbSelected( false );
				this.bombEditor.setVectorBrick( this.vBrick );
			}
			this.setbHud( false );
			this.panel.textX.setText( "" );
			this.panel.textY.setText( "" );
			this.panel.textX.setVisible( false );
			this.panel.textY.setVisible( false );
			this.selected	= -1;
			return;
		}
		if ( this.selected != minusSelected )
			return;
		if ( minusSelected > 3000 )
		{
			this.bombEditor.setVectorBrick( this.vBrick );
			if ( minusSelected != 3001 )
				this.selected--;
			else
			{
				if ( this.vBrick.size() > 0 )
					this.selected = 3001;
				else if ( this.vSweety.size() > 0 )
					this.selected	= 1000 + this.vSweety.size();
				else if ( this.vFinish.size() > 0 )
					this.selected = 2001;
				else if ( this.vWall.size() > 0 )
					this.selected	= 1;
				else
					this.selected	= -1;
			}
		} else if ( minusSelected > 2000 )
		{
			this.bombEditor.setVectorFinish( this.vFinish );
			if ( minusSelected != 2001 )
			{
				this.selected--;
			} else
			{
				if ( this.vFinish.size() > 0 )
					this.selected = 2001;
				else if ( this.vSweety.size() > 0 )
					this.selected	= 1000 + this.vSweety.size();
				else if ( this.vWall.size() > 0 )
					this.selected	= this.vWall.size();
				else if ( this.vBrick.size() > 0 )
					this.selected = 3001;
				else
					this.selected	= -1;
			}
		} else if ( minusSelected > 1000 )
		{
			this.bombEditor.setVectorSweety( this.vSweety );
			if ( minusSelected != 1001 )
				this.selected--;
			else
			{
				if ( this.vSweety.size() > 0 )
					this.selected	= 1000 + this.vSweety.size();
				else if ( this.vFinish.size() > 0 )
					this.selected = 2001;
				else if ( this.vWall.size() > 0 )
					this.selected	= this.vWall.size();
				else if ( this.vBrick.size() > 0 )
					this.selected = 3001;
				else
					this.selected	= -1;
			}
		} else
		{
			this.bombEditor.setVectorWall( this.vWall );
			if ( minusSelected != 1 )
				this.selected--;
			else
			{
				if ( this.vWall.size() > 0 )
					this.selected	= 1;
				else if ( this.vBrick.size() > 0 )
					this.selected = 3001;
				else if ( this.vSweety.size() > 0 )
					this.selected	= 1000 + this.vSweety.size();
				else if ( this.vFinish.size() > 0 )
					this.selected = 2001;
				else
					this.selected	= -1;
			}
		}
		//System.out.println(this.selected+" und geloescht "+minusSelected);
		this.setSelected( this.selected );
	}
	
	private void setNextSelected()
	{
		int next	= this.selected;
		if ( this.selected <= 0 )
		{
			if ( this.vWall.size() != 0 )
				next	= 1;
			else if ( this.vSweety.size() != 0 )
				next	= 1001;
			else if ( this.vFinish.size() != 0 )
				next	= 2001;
			else if ( this.vBrick.size() != 0 )
				next	= 3001;
			else
				return;
		} else if ( this.selected < 1000 )
		{
			if ( this.selected < this.vWall.size() )
				next++;
			else
			{
				if ( this.vBrick.size() > 0 )
					next = 3001;
				else if ( this.vSweety.size() > 0 )
					next	= 1001;
				else if ( this.vFinish.size() > 0 )
					next = 2001;
				else if ( this.vWall.size() > 0 )
					next = 1;
			}
		} else if ( this.selected < 2000 )
		{
			if ( this.selected < 1000 + this.vSweety.size() )
				next++;
			else
			{
				if ( this.vFinish.size() > 0 )
					next = 2001;
				else if ( this.vWall.size() > 0 )
					next	= 1;
				else if ( this.vBrick.size() > 0 )
					next = 3001;
				else if ( this.vSweety.size() > 0 )
					next	= 1001;
			}
		} else if ( this.selected < 3000 )
		{
			if ( this.selected < 2000 + this.vFinish.size() )
				next++;
			else
			{
				if ( this.vWall.size() > 0 )
					next	= 1;
				else if ( this.vBrick.size() > 0 )
					next = 3001;
				else if ( this.vSweety.size() > 0 )
					next = 1001;
				else if ( this.vFinish.size() > 0 )
					next = 2001;
			}
		} else
		{
			if ( this.selected < 3000 + this.vBrick.size() )
				next++;
			else
			{
				if ( this.vSweety.size() > 0 )
					next	= 1001;
				else if ( this.vFinish.size() > 0 )
					next = 2001;
				else if ( this.vWall.size() > 0 )
					next = 1;
				else if ( this.vBrick.size() > 0 )
					next = 3001;
			}
		}
		this.setSelected( next );
	}
	
	private void setPreviousSelected()
	{
		int next	= this.selected;
		if ( this.selected <= 0 )
		{
			if ( this.vFinish.size() != 0 )
				next	= 2000 + this.vFinish.size();
			else if ( this.vSweety.size() != 0 )
				next	= 1000 + this.vSweety.size();
			else if ( this.vBrick.size() != 0 )
				next	= 3000 + this.vBrick.size();
			else if ( this.vWall.size() != 0 )
				next	= 0 + this.vWall.size();
			else
				return;
		} else if ( this.selected < 1000 )
		{
			if ( this.selected > 1 )
				next--;
			else
			{
				if ( this.vFinish.size() > 0 )
					next = 2000 + this.vFinish.size();
				else if ( this.vSweety.size() > 0 )
					next = 1000 + this.vSweety.size();
				else if ( this.vBrick.size() > 0 )
					next = 3000 + this.vBrick.size();
				else if ( this.vWall.size() > 0 )
					next = this.vWall.size();
			}
		} else if ( this.selected < 2000 )
		{
			if ( this.selected > 1001 )
				next--;
			else
			{
				if ( this.vBrick.size() > 0 )
					next = 3000 + this.vBrick.size();
				else if ( this.vWall.size() > 0 )
					next	= this.vWall.size();
				else if ( this.vFinish.size() > 0 )
					next = 2000 + this.vFinish.size();
				else if ( this.vSweety.size() > 0 )
					next = 1000 + this.vSweety.size();
			}
		} else if ( this.selected < 3000 )
		{
			if ( this.selected > 2001 )
				next--;
			else
			{
				if ( this.vSweety.size() > 0 )
					next = 1000 + this.vSweety.size();
				else if ( this.vBrick.size() > 0 )
					next = 3000 + this.vBrick.size();
				else if ( this.vWall.size() > 0 )
					next	= this.vWall.size();
				else if ( this.vFinish.size() > 0 )
					next = 2000 + this.vFinish.size();
			}
		} else
		{
			if ( this.selected > 3001 )
				next--;
			else
			{
				if ( this.vWall.size() > 0 )
					next = this.vWall.size();
				else if ( this.vFinish.size() > 0 )
					next = 2000 + this.vFinish.size();
				else if ( this.vSweety.size() > 0 )
					next = 1000 + this.vSweety.size();
				else if ( this.vBrick.size() > 0 )
					next = 3000 + this.vBrick.size();
			}
		}
		this.setSelected( next );
	}
	
	/**
	 * Methode zum uebernehmen der eingegebenen Werte
	 */
	private void setValue()
	{
		if ( this.selected <= 0 )
			return;
		int xValue, yValue, width, height;
		try
		{
			xValue = Integer.parseInt( this.panel.textX.getText());
			if ( xValue < 0 )
				xValue	= 0;
			else if ( xValue > 616 )
				xValue	= 616;
			this.panel.textX.setText( ""+xValue );
		}
		catch (NumberFormatException e)
		{
			if ( this.selected < 1000 )
				xValue = (this.vWall.get( this.selected - 1 )).getX();
			else if ( this.selected < 2000 )
				xValue = (int)(this.vSweety.get( -1000 + this.selected - 1 )).getX();
			else if ( this.selected < 3000 )
				xValue = (this.vFinish.get( -2000 + this.selected - 1 )).getX();
			else
				xValue = (this.vBrick.get( -3000 + this.selected - 1 )).getX();
		}
		try
		{
			yValue = Integer.parseInt( this.panel.textY.getText());
			if ( yValue < 0 )
				yValue	= 0;
			else if ( yValue > 376 )
				yValue	= 376;
			this.panel.textY.setText( ""+yValue );
		}
		catch (NumberFormatException e)
		{
			if ( this.selected < 1000 )
				yValue = (this.vWall.get( this.selected - 1 )).getY();
			else if ( this.selected < 2000 )
				yValue = (int)(this.vSweety.get( -1000 + this.selected - 1 )).getY();
			else if ( this.selected < 3000 )
				yValue = (this.vFinish.get( -2000 + this.selected - 1 )).getY();
			else
				yValue = (this.vBrick.get( -3000 + this.selected - 1 )).getY();
		}
		
		if ( ( this.selected > 2000 ) && ( this.selected < 3000 ) )
		{
			try
			{
				height = Integer.parseInt( this.panel.textHeight.getText());
				if ( height < 1 )
					height	= 1;
				else if ( height + yValue > 400 )
					height	= 400 - yValue;
				this.panel.textHeight.setText( ""+height );
			}
			catch (NumberFormatException e)
			{
				height = (this.vFinish.get( -2000 + this.selected - 1 )).getHeight();
			}
			try
			{
				width = Integer.parseInt( this.panel.textWidth.getText());
				if ( width < 1 )
					width	= 1;
				else if ( width + xValue > 640 )
					width	= 640 - xValue;
				this.panel.textWidth.setText( ""+width );
			}
			catch (NumberFormatException e)
			{
				width = (this.vFinish.get( -2000 + this.selected - 1 )).getWidth();
			}
			(this.vFinish.get( -2000 + this.selected - 1 )).setX( xValue );
			(this.vFinish.get( -2000 + this.selected - 1 )).setY( yValue );
			(this.vFinish.get( -2000 + this.selected - 1 )).setHeight( height );
			(this.vFinish.get( -2000 + this.selected - 1 )).setWidth( width );
			this.bombEditor.setVectorFinish( this.vFinish );
		} else
		{
			if ( this.selected < 1000 )
			{
				(this.vWall.get( this.selected - 1 )).setX( xValue );
				(this.vWall.get( this.selected - 1 )).setY( yValue );
				this.bombEditor.setVectorWall( this.vWall );
			} else if ( this.selected < 2000 )
			{
				(this.vSweety.get( -1000 + this.selected - 1 )).setX( xValue );
				(this.vSweety.get( -1000 + this.selected - 1 )).setY( yValue );
				this.bombEditor.setVectorSweety( this.vSweety );
			} else
			{
				(this.vBrick.get( -3000 + this.selected - 1 )).setX( xValue );
				(this.vBrick.get( -3000 + this.selected - 1 )).setY( yValue );
				this.bombEditor.setVectorBrick( this.vBrick );
			}
		}
	}
	
	private void setDelete()
	{
		if ( this.selected <= 0 )
			return;
		if ( this.selected < 1000 )
		{
			this.vWall.remove( this.selected - 1 );
			this.bombEditor.setVectorWall( this.vWall );
		} else if ( this.selected < 2000 )
		{
			this.vSweety.remove( this.selected - 1 - 1000 );
			this.bombEditor.setVectorSweety( this.vSweety );
		} else if ( this.selected < 3000 )
		{
			this.vFinish.remove( this.selected - 1 - 2000 );
			this.bombEditor.setVectorFinish( this.vFinish );
		} else
		{
			this.vBrick.remove( this.selected - 1 - 2000 );
			this.bombEditor.setVectorBrick( this.vBrick );
		}
		this.setDeleteSelected( this.selected );
	}
	
	private void setChooseUp( boolean bDelete )
	{
		int old			= this.level;
		int bombCount	= 0;
		try
		{
			bombCount = Integer.parseInt( this.panel.textBomb.getText());
		}
		catch (NumberFormatException e)
		{
			bombCount = 5;
		}
		this.level--;
		if ( this.level < 0 )
			this.level	= this.bombLevel.getLevelCount() - 1;
		if ( ( ( old != this.level ) && ( this.bombLevel.getLevelCount() != 1 ) ) || ( bDelete ) )
		{
			this.selected	= -1;
			this.panel.textX.setVisible( false );
			this.panel.textY.setVisible( false );
			this.bombLevel.setAll( this.vWall, this.vBrick, this.vSweety, this.vFinish, bombCount, this.panel.textName.getText(), old );
			this.vWall		= this.bombLevel.getWallVector( this.level );
			this.vSweety	= this.bombLevel.getSweetyVector( this.level );
			this.vFinish	= this.bombLevel.getFinishVector( this.level );
			this.vBrick		= this.bombLevel.getBrickVector( this.level );
			this.panel.textBomb.setText( ""+this.bombLevel.getBombCount( this.level ) );
			this.panel.textName.setText( ""+this.bombLevel.getName( this.level ) );
			this.bombEditor.loadChoose( this.vWall, this.vBrick, this.vSweety, this.vFinish );
		}
	}
	
	private void setChooseDown()
	{
		int old			= this.level;
		int bombCount	= 0;
		try
		{
			bombCount = Integer.parseInt( this.panel.textBomb.getText());
		}
		catch (NumberFormatException e)
		{
			bombCount = 5;
		}
		this.level++;
		if ( this.level >= this.bombLevel.getLevelCount() )
			this.level	= 0;
		if ( ( old != this.level ) && ( this.bombLevel.getLevelCount() != 1 ) )
		{
			this.selected	= -1;
			this.panel.textX.setVisible( false );
			this.panel.textY.setVisible( false );
			this.bombLevel.setAll( this.vWall, this.vBrick, this.vSweety, this.vFinish, bombCount, this.panel.textName.getText(), old );
			this.vWall		= this.bombLevel.getWallVector( this.level );
			this.vSweety	= this.bombLevel.getSweetyVector( this.level );
			this.vFinish	= this.bombLevel.getFinishVector( this.level );
			this.vBrick		= this.bombLevel.getBrickVector( this.level );
			this.panel.textBomb.setText( ""+this.bombLevel.getBombCount( this.level ) );
			this.panel.textName.setText( ""+this.bombLevel.getName( this.level ) );
			this.bombEditor.loadChoose( this.vWall, this.vBrick, this.vSweety, this.vFinish );
		}
	}
	
	private void setChangeUp()
	{
		int old			= this.level;
		int bombCount	= 0;
		try
		{
			bombCount = Integer.parseInt( this.panel.textBomb.getText());
		}
		catch (NumberFormatException e)
		{
			bombCount = 5;
		}
		this.level--;
		if ( this.level < 0 )
			this.level	= this.bombLevel.getLevelCount() - 1;
		if ( ( old != this.level ) && ( this.bombLevel.getLevelCount() != 1 ) )
		{
			this.bombLevel.setAll( this.vWall, this.vBrick, this.vSweety, this.vFinish, bombCount, this.panel.textName.getText(), old );
			this.bombLevel.changeLevel( old, this.level );
			this.bombEditor.setBombLevel( this.bombLevel );
			this.setAllVector();
			this.panel.textBomb.setText( ""+this.bombLevel.getBombCount( this.level ) );
			this.panel.textName.setText( ""+this.bombLevel.getName( this.level ) );
		}
		
	}
	
	private void setChangeDown()
	{
		int old			= this.level;
		int bombCount	= 0;
		try
		{
			bombCount = Integer.parseInt( this.panel.textBomb.getText());
		}
		catch (NumberFormatException e)
		{
			bombCount = 5;
		}
		this.level++;
		if ( this.level >= this.bombLevel.getLevelCount() )
			this.level	= 0;
		if ( ( old != this.level ) && ( this.bombLevel.getLevelCount() != 1 ) )
		{
			this.bombLevel.setAll( this.vWall, this.vBrick, this.vSweety, this.vFinish, bombCount, this.panel.textName.getText(), old );
			this.bombLevel.changeLevel( old, this.level );
			this.bombEditor.setBombLevel( this.bombLevel );
			this.setAllVector();
			this.panel.textBomb.setText( ""+this.bombLevel.getBombCount( this.level ) );
			this.panel.textName.setText( ""+this.bombLevel.getName( this.level ) );
		}
		
	}
	
	private void setNewLevel()
	{
		if ( this.bombLevel != null )
		{
			this.selected	= -1;
			this.bombLevel.setNewLevel();
		}
	}
	
	private void setDeleteLevel()
	{
		if ( ( this.bombLevel != null ) && ( this.bombLevel.getLevelCount() != 1 ) )
		{
			int old		= this.level;
			this.setChooseUp( true );
			this.bombLevel.setDelete( old );
			this.bombEditor.setBombLevel( this.bombLevel );
			if ( this.level >= this.bombLevel.getLevelCount() )
				this.level	= this.bombLevel.getLevelCount()-1;
		}
	}
	
	public void setText( BombLevel bombLevel )
	{
		this.panel.textBomb.setText( ""+bombLevel.getBombCount( this.level ) );
		this.panel.textName.setText( ""+bombLevel.getName( this.level ) );
	}
	
	public void mouseReleased( int x, int y )
	{
		for (int i = 0; i < Buttons.length; i++)
		{
			if ( (Buttons[i].getbPressed()) && (Buttons[i].getx() <= x) &&
				 (Buttons[i].getx() + Buttons[i].getWidth() > x) && (Buttons[i].gety() <= y) &&
				 (Buttons[i].gety() + Buttons[i].getHeight() > y) && (Buttons[i].getbVisible()))
			{
				Buttons[i].setbPressed( false );
				if ( "Menu".equals( Buttons[i].getFunction() ) )
				{
					this.bombEditor.setMenu();
					return;
				} else if ("Sweety".equals( Buttons[i].getFunction() ))
				{
					this.bombEditor.setNewSweety();
					this.vSweety		= this.bombEditor.getvSweety();
				} else if ("Wall".equals( Buttons[i].getFunction() ))
				{
					this.bombEditor.setNewWall();
					this.vWall		= this.bombEditor.getvWall();
				} else if ("Brick".equals( Buttons[i].getFunction() ))
				{
					this.bombEditor.setNewBrick();
					this.vBrick		= this.bombEditor.getvBrick();
				} else if ("Finish".equals( Buttons[i].getFunction() ))
				{
					this.bombEditor.setNewFinish( true );
					this.vFinish	= this.bombEditor.getvFinish();
				} else if ("Save".equals( Buttons[i].getFunction() ))
				{
					int bombCount;
					try
					{
						bombCount = Integer.parseInt( this.panel.textBomb.getText());
					}
					catch (NumberFormatException e)
					{
						bombCount = 5;
					}
					if ( bombCount < 1 )
						bombCount = 1;
					else if ( bombCount > 5 )
						bombCount = 5;
					if ( this.panel.textName.getText().length() == 0 )
						this.panel.textName.setText( "Apo" );
					else if ( this.panel.textName.getText().length() > 10 )
						this.panel.textName.setText( this.panel.textName.getText().substring( 0, 10 ) );
					this.bombEditor.save( this.vWall, this.vBrick, this.vSweety, this.vFinish, bombCount, this.panel.textName.getText(), this.level );
					this.bombLevel		= this.bombEditor.getBombLevel();
				} else if ("Load".equals( Buttons[i].getFunction() ))
				{
					this.level		= 0;
					this.bombEditor.load( this.level );
					this.bombLevel		= this.bombEditor.getBombLevel();
					
				} else if ("Ok".equals( Buttons[i].getFunction() ))
				{
					this.setValue();
				} else if ("Delete".equals( Buttons[i].getFunction() ))
				{
					this.setDelete();
				} else if ("Left".equals( Buttons[i].getFunction() ))
				{
					this.setPreviousSelected();
				} else if ("Right".equals( Buttons[i].getFunction() ))
				{
					this.setNextSelected();
				} else if ("UpChoose".equals( Buttons[i].getFunction() ))
				{
					this.setChooseUp( false );
				} else if ("UpChange".equals( Buttons[i].getFunction() ))
				{
					this.setChangeUp();
				} else if ("DownChoose".equals( Buttons[i].getFunction() ))
				{
					this.setChooseDown();
				} else if ("DownChange".equals( Buttons[i].getFunction() ))
				{
					this.setChangeDown();
				} else if ("NewLevel".equals( Buttons[i].getFunction() ))
				{
					this.setNewLevel();
				} else if ("DeleteLevel".equals( Buttons[i].getFunction() ))
				{
					this.setDeleteLevel();
				}
				break;
			} else
			{
				Buttons[i].setbOver( false );
				Buttons[i].setbPressed( false );
			}
		}
	}
	
	public void mousePressed( int x, int y )
	{
		for (int i = 0; i < Buttons.length; i++)
		{
			if ( (Buttons[i].getbVisible()) && (Buttons[i].getbOver()) && (Buttons[i].getx() <= x) &&
				 (Buttons[i].getx() + Buttons[i].getWidth() > x) && (Buttons[i].gety() <= y) &&
				 (Buttons[i].gety() + Buttons[i].getHeight() > y))
			{
				Buttons[i].setbPressed( true );
				break;
			}
		}
		this.panel.mousePressed( x, y );
	}
	
	public void mouseMoved( int x, int y )
	{
		for (int i = 0; i < Buttons.length; i++)
		{
			if ( (Buttons[i].getbVisible()) && (!Buttons[i].getbOver()) && (Buttons[i].getx() <= x) &&
				 (Buttons[i].getx() + Buttons[i].getWidth() > x) && (Buttons[i].gety() <= y) &&
				 (Buttons[i].gety() + Buttons[i].getHeight() > y))
			{
				Buttons[i].setbOver( true );
			} else if (	(Buttons[i].getbOver()) && ((Buttons[i].getx() > x) ||
					 	(Buttons[i].getx() + Buttons[i].getWidth() <= x) || (Buttons[i].gety() > y) ||
					 	(Buttons[i].gety() + Buttons[i].getHeight() <= y)) )
			{
				Buttons[i].setbOver( false );
				Buttons[i].setbPressed( false );
			}
		}
	}

	public void keyReleased( int key, char character )
	{
		this.panel.keyReleased( key, character );
	}
	
	public void render(Graphics2D gfx) 
	{
		gfx.drawImage( this.iBackground,     0, 400, null );
		gfx.drawImage( this.iMiddle,        80, 403, null );
		gfx.drawImage( this.iMiddleBomb,   283, 417, null );
		gfx.setColor( Color.BLACK );
		if ( this.bHud )
		{
			gfx.drawString( "W = ", 90, 470 );
			gfx.drawString( "= H", 250, 470 );
		}
		if ( this.selected > 0 )
		{
			gfx.drawString( "X = ", 90, 445 );
			gfx.drawString( "= Y", 250, 445 );
			if ( this.selected >= 3000 )
				gfx.drawImage( this.iBrick, 168, 408, null );
			else if ( this.selected >= 2000 )
				gfx.drawImage( this.iFinish, 168, 408, null );
			else if ( this.selected >= 1000 )
				gfx.drawImage( this.iSweety, 168, 408, 192, 432, 0, 0, 32, 32, null );
			else
				gfx.drawImage( this.iWall, 168, 408, null );
		}
		this.panel.render( gfx );
		this.drawChoose( gfx );
		for ( int i = 0; i < this.Buttons.length; i++ )
		{
			this.Buttons[i].render( gfx );
		}
	}
	
	private void drawChoose( Graphics2D gfx )
	{
		gfx.drawImage( this.iMiddleChoose, 410, 405, null );
		if ( this.bombLevel != null )
		{
			int count	= this.bombLevel.getLevelCount();
			int start	= this.level;
			if ( start + 4 >= count )
				start	= count - 4;
			if ( start < 0 )
				start	= 0;
			int y		= 0;
			for ( int i = start; i < start + 4; i++ )
			{
				if ( this.bombLevel.getLevelCount() <= i )
					break;
				if ( this.level == i )
					gfx.drawImage( this.iMiddleChooseChoose, 413, 408 + y, null );
				String s	= this.bombLevel.getName( i );
				if ( s.length() > 12 )
					s	= s.substring( 0, 12 );
				gfx.drawString( s, 415, 420 + y );
				y		+= 15;
			}
		}
	}

}