package bomb;

import java.awt.Dimension;
import java.awt.Graphics2D;

public class BombGame implements BombInterface
{
	private static final long serialVersionUID = 1L;
	
	/** The time at which the last rendering looped started from the point of view of the game logic */
	private long lastLoopTime = BombSystemTimer.getTime();
	/** The time since the last record of fps */
	private long lastFpsTime = 0;
	/** The recorded fps */
	private int fps;
	
	private BombMain 		bombMain;
	
	private BombGameGround	bombGameGround;

	private BombGameHud		bombGameHud;
	
	private BombLevel		bombLevel;
	
	private BombThreadLogic	bombThreadLogic;
	
	private boolean			bLogic;
	
	private int				loadLevel, level;
	
	public BombGame( BombMain bombMain, BombImage bombImage )
	{
		super();
		
		this.bombMain		= bombMain;
		
		this.bombGameGround	= new BombGameGround( this, bombImage );
		
		this.bombGameHud	= new BombGameHud( this, bombImage );
		
		this.bLogic			= false;
		this.loadLevel		= -1;
		this.level			= 0;
		
		this.bombThreadLogic	= new BombThreadLogic( this, 10 );
	}
	
	public Dimension getPreferredSize()
	{
		return new Dimension( 640, 480);
	}
	
	public boolean getRenderAble()
	{
		return false;
	}

	public boolean getLogicAble()
	{
		return this.bLogic;
	}

	public void frameRenderer( Graphics2D g )
	{
		BombSystemTimer.sleep(lastLoopTime+10-BombSystemTimer.getTime());
		
		// work out how long its been since the last update
		long delta = BombSystemTimer.getTime() - lastLoopTime;
		lastLoopTime = BombSystemTimer.getTime();
		lastFpsTime += delta;
		fps++;
		
		// update our FPS counter if a second has passed
		if ( this.lastFpsTime >= 1000)
		{
			this.bombGameGround.setFPS( "fps: "+fps );
			this.lastFpsTime 	= 0;
			this.fps 			= 0;
		}
		
		this.loadLevel();
		
		this.bombGameGround.render( g );
		this.bombGameHud.render( g );
	}

	public void logic()
	{
		if ( this.getLogicAble() )
		{
			this.bombGameGround.think( 10 );
		}
	}
	
	private void loadLevel()
	{
		if ( this.loadLevel != -1 )
		{
			if ( this.loadLevel == 1 )
			{
				BombLevel bombLevel	= this.bombMain.getBombLevelGame();
				this.bombGameHud.setNewLevel( bombLevel, 0 );
				this.bombGameGround.setNewLevel( bombLevel, 0 );
				this.bombGameGround.setBombArray( this.bombGameHud.getBombArray() );
			} else if ( this.loadLevel == 2 )
			{
				this.bombGameHud.setNewLevel( bombLevel, level );
				this.bombGameGround.setNewLevel( bombLevel, level );
				this.bombGameGround.setBombArray( this.bombGameHud.getBombArray() );
			} else if ( this.loadLevel == 3 )
			{
				this.bombGameHud.setNewLevel( bombLevel, level );
				this.bombGameGround.setBombArray( this.bombGameHud.getBombArray() );
			}
			this.loadLevel	= -1;
		}
	}
	
	public void setMenu()
	{
		this.bombMain.setMenu();
	}
	
	public BombGameGround getBombGameGround()
	{
		return this.bombGameGround;
	}
	
	public void setThreadStart()
	{
		this.bombGameGround.setThreadStart();
		this.bLogic			= true;
	}
	
	public void setThreadStop()
	{
		this.bombThreadLogic.stop();
		this.bLogic			= false;
		this.bombGameGround.setThreadStop( true );
	}
	
	public void setbThread( boolean bThread )
	{
		this.bombGameHud.setbThread( bThread );
		if ( bThread )
		{
			this.bombThreadLogic.start();
			this.bLogic			= true;
		} else
		{
			this.bombThreadLogic.stop();
			this.bLogic			= false;
		}
	}
	
	public void setBombSelect( int bombSelect )
	{
		this.bombGameHud.setBombSelect( bombSelect );
	}
	
	public void setbWin( boolean bWin )
	{
		this.bombGameHud.setbWin( bWin );
	}

	public void setKeyStart( boolean bStart )
	{
		this.bombGameHud.setStart( bStart );
	}
	
	public void load()
	{
		this.bombMain.load( false );
		this.loadLevel	= 1;
	}
	
	public void setLoadLevel( BombLevel bombLevel, int i )
	{
		this.bombLevel	= bombLevel;
		this.level		= i;
		this.loadLevel	= 2;
	}
	
	public void setNewBomb( BombLevel bombLevel, int level )
	{
		this.bombLevel	= bombLevel;
		this.level		= level;
		this.loadLevel	= 3;
	}
	
	public void think( int delay )
	{
		this.bombGameGround.think( delay );
	}
	
	public void mousePressed( int x, int y )
	{
		if ( y < 400 )
			this.bombGameGround.mousePressed( x, y );
		else
			this.bombGameHud.mousePressed( x, y );
	}
	
	public void mouseReleased( int x, int y )
	{
		if ( y < 400 )
			this.bombGameGround.mouseReleased( x, y );
		else
			this.bombGameHud.mouseReleased( x, y );
	}
	
	public void mouseReleasedRight( int x, int y )
	{
		if ( y < 400 )
			this.bombGameGround.mouseReleasedRight( x, y );
	}
	
	public void mouseMoved( int x, int y )
	{
		if ( y < 400 )
			this.bombGameGround.mouseMove( x, y );
		else
			this.bombGameHud.mouseMove( x, y );
	}
	
	public void mouseDragged( int x, int y )
	{
		if ( y < 400 )
			this.bombGameGround.mouseDragged( x, y );
		else
			this.bombGameHud.mouseDragged( x );
	}
	
	public void keyReleased( int key )
	{
		this.bombGameGround.keyReleased( key );
	}

}
