package bomb;

import java.awt.Color;
import java.awt.Font;
import java.awt.Graphics2D;
import java.awt.image.BufferedImage;
import java.io.File;
import java.util.Vector;
import java.util.concurrent.TimeUnit;

public class BombGameGround
{
	private BombGame			bombGame;
	
	private BombImage			bombImage;
	
	private BufferedImage		iBackground, iSweetyNormal, iSweetyTime, 
								iTimeImage, iWin, iSweetyExplode, iLevel;

	private BombSweety[]		sweetyArray;
	private BombBombe[]			bombArray;
	private BombWall[]			wallArray;
	private BombBrick[]			brickArray;
	private BombFinish[]		finishArray;

	private BombSounds[]		aSoundScream;

	private BombLevel			bL;

	private int					level;

	private long 				time;
	
	private Font				FontMiddle;

	private String				fps;

	private boolean				bThread, bWin;
	
	public BombGameGround( BombGame	bombGame, BombImage	bombImage )
	{
		super();
		
		this.bombGame		= bombGame;
		this.bombImage		= bombImage;
		
		this.iBackground		= this.bombImage.setPicsMain( "/images/game_background.jpg", false );
		this.iSweetyNormal		= this.bombImage.setPicsMain( "/images/Sweety_Normal.png", false );
		this.iSweetyTime		= this.bombImage.setPicsMain( "/images/Sweety_Bomb.png", false );
		this.iSweetyExplode		= this.bombImage.setPicsMain( "/images/Sweety_Tod.png", false );
		this.iTimeImage			= this.bombImage.setPicsMain( "/images/Time.png", false );
		this.iWin				= this.bombImage.setPicsMain( "/images/game_ground_win.png", false );
		this.iLevel				= this.bombImage.setPicsMain( "/images/game_ground_level.png", false );
		
		this.sweetyArray		= new BombSweety[1];
		this.sweetyArray[0]		= new BombSweety( 50.0D, 40.0D, this.iSweetyNormal.getWidth(), this.iSweetyNormal.getHeight(), this.iSweetyNormal, this.iSweetyTime, this.iSweetyExplode );
		
		this.aSoundScream		= new BombSounds[16];
		for ( int i = 0; i < this.aSoundScream.length; i++ )
		{
			this.aSoundScream[i]	= new BombSounds( new File( System.getProperty("user.dir") + File.separator+"sounds" + File.separator + "Scream" + (i+1) + ".wav") );
		}
		
		this.sweetyArray[0].setSoundScream( this.aSoundScream );
		
		this.time				= 0;
		this.level				= 0;
		
		this.FontMiddle			= new Font( "Dialog", Font.BOLD, 19 );
        
        this.fps				= "";
        
        this.bWin				= false;
	}
	
	public void setNewLevel( BombLevel bL, int level )
	{
		this.bL				= bL;
		this.level			= level;
		if ( bL.getWall( this.level ) == null )
			return;
		this.wallArray		= bL.getWall( level );
		this.brickArray		= bL.getBrick( level );
		this.sweetyArray	= bL.getSweety( level );
		for ( int i = 0; i < this.sweetyArray.length; i++ )
		{
			this.sweetyArray[i].setSoundScream( this.aSoundScream );
		}
		this.finishArray	= bL.getFinish( level );
	}
	
	public boolean getbThread()
	{
		return this.bThread;
	}
	
	public void setThreadStart()
	{
        this.bThread			= true;
        this.bombGame.setbThread( this.bThread );
        this.time				= System.nanoTime();
	}
	
	public void setThreadStop( boolean bRefresh )
	{
		this.bThread			= false;
		this.bombGame.setbThread( this.bThread );
		this.bombGame.setbWin( this.bWin );
		this.fps				= "";
		if ( bRefresh )
		{
			for ( int i = 0; i < this.sweetyArray.length; i++ )
			{
				this.sweetyArray[i].setReset();
			}
			for ( int i = 0; i < this.bombArray.length; i++ )
			{
				this.bombArray[i].setReset();
			}
			for ( int i = 0; i < this.brickArray.length; i++ )
			{
				this.brickArray[i].setReset();
			}
		}
		this.time				= 0;
	}
	
	public void setBombArray( BombBombe[] bombArray )
	{
		this.bombArray	= bombArray;
	}
	
	public void think( int INCTIME )
	{
		//this.time		+= INCTIME;
		this.setKollisionCheck();
		this.setBombCheck();
		if (this.setOutCheck() == false)
		{
			this.setSpeedCheck();
		}
	}
	
	@SuppressWarnings("unchecked")
	private void setKollisionCheck()
	{
		for ( int i = 0; i < this.sweetyArray.length; i++ )
		{
			this.sweetyArray[i].think();
			double ySpeed	= this.sweetyArray[i].getYSpeed();
			double xSpeed	= this.sweetyArray[i].getXSpeed();
			Vector	vBrick	= new Vector();
			for ( int j = 0; j < this.brickArray.length; j++ )
			{
				if ( ( this.brickArray[j].getbBrick() ) && ( this.brickArray[j].getbWallHit( this.sweetyArray[i].getX(), this.sweetyArray[i].getY(), this.sweetyArray[i].getWidth(), this.sweetyArray[i].getHeight() ) ) )
				{
					vBrick.add( j );
				}
				if ( ( !this.brickArray[j].getbBrick() ) && ( this.brickArray[j].getbVisible() ) )
					this.brickArray[j].setExplode();
			}
			Vector	vWall	= new Vector();
			for ( int j = 0; j < this.wallArray.length; j++ )
			{
				if ( this.wallArray[j].getbWallHit( this.sweetyArray[i].getX(), this.sweetyArray[i].getY(), this.sweetyArray[i].getWidth(), this.sweetyArray[i].getHeight() ) )
				{
					vWall.add( j );
				}
			}
			if ( ( vWall.size() == 0 ) && ( vBrick.size() == 0 ) && ( this.sweetyArray[i].getYSpeed() == 0 ) )
			{
				this.sweetyArray[i].setYSpeed();
			}
			this.setBrickCheckMiddle( i, xSpeed, ySpeed, vBrick );
			boolean bWallMiddle	= this.setWallCheckMiddle( i, vWall );
			this.setBrickCheckCorner( i, xSpeed, ySpeed );
			if ( !bWallMiddle )
				this.setWallCheckCorner( i );
		}
	}
	
	@SuppressWarnings("unchecked")
	private boolean setWallCheckMiddle( int i, Vector vWall )
	{
		if ( ( this.wallArray == null ) || ( this.wallArray.length <= 0 ) )
			return false;
		if ( vWall.size() != 0 )
		{
			String s	= "";
			int contactWall	= -1;
			for ( int j = 0; j < vWall.size(); j++ )
			{
				s	= this.wallArray[ ((Integer)vWall.get( j )).intValue() ].getMiddleHindrance( this.sweetyArray[i].getX(), this.sweetyArray[i].getY(), this.sweetyArray[i].getWidth(), this.sweetyArray[i].getHeight() );
				if ( !s.equals("") )
				{
					contactWall	= ((Integer)vWall.get( j )).intValue();
					break;
				}
			}
			if ( contactWall != -1 )
			{
				this.sweetyArray[i].setHindrance( s, this.wallArray[ contactWall ].getX(), this.wallArray[ contactWall ].getY(), this.wallArray[ contactWall ].getWidth(), this.wallArray[ contactWall ].getHeight() );
				return true;
			}
		}
		return false;
	}
	
	@SuppressWarnings("unchecked")
	private boolean setWallCheckCorner( int i )
	{
		Vector	vWall	= new Vector();
		if ( ( this.wallArray == null ) || ( this.wallArray.length <= 0 ) )
			return false;
		for ( int j = 0; j < this.wallArray.length; j++ )
		{
			if ( this.wallArray[j].getbWallHit( this.sweetyArray[i].getX(), this.sweetyArray[i].getY(), this.sweetyArray[i].getWidth(), this.sweetyArray[i].getHeight() ) )
			{
				vWall.add( j );
			}
		}
		String s	= "";
		int contactWall	= -1;
		for ( int j = 0; j < vWall.size(); j++ )
		{
			s	= this.wallArray[ ((Integer)vWall.get( j )).intValue() ].getCornerHindrance( this.sweetyArray[i].getX(), this.sweetyArray[i].getY(), this.sweetyArray[i].getWidth(), this.sweetyArray[i].getHeight(), this.sweetyArray[i].getXSpeed(), this.sweetyArray[i].getYSpeed() );
			if ( !s.equals("") )
			{
				contactWall	= ((Integer)vWall.get( j )).intValue();
				break;
			}
		}
		if ( contactWall != -1 )
		{
			this.sweetyArray[i].setHindrance( s, this.wallArray[ contactWall ].getX(), this.wallArray[ contactWall ].getY(), this.wallArray[ contactWall ].getWidth(), this.wallArray[ contactWall ].getHeight() );
			return true;
		}
		return false;
	}
	
	@SuppressWarnings("unchecked")
	private boolean setBrickCheckMiddle( int i, double xSpeed, double ySpeed, Vector vBrick )
	{
		if ( vBrick.size() != 0 )
		{
			Vector vString 	= new Vector();
			Vector vWall	= new Vector();
			for ( int j = 0; j < vBrick.size(); j++ )
			{
				if ( this.brickArray[((Integer)vBrick.get( j )).intValue()].getbBrick() )
				{
					String s;
					s	= this.brickArray[ ((Integer)vBrick.get( j )).intValue() ].getMiddleHit( this.sweetyArray[i].getX(), this.sweetyArray[i].getY(), this.sweetyArray[i].getWidth(), this.sweetyArray[i].getHeight(), xSpeed, ySpeed );
					if ( !s.equals("") )
					{
						vString.add( s );
						vWall.add(((Integer)vBrick.get( j )).intValue());
					}
				}
			}
			if ( vString.size() != 0 )
			{
				for ( int j = 0; j < vString.size(); j++ )
				{
					this.sweetyArray[i].setHindrance( vString.get( j ).toString(), this.brickArray[ ((Integer)(vWall.get( j ))).intValue() ].getX(), this.brickArray[ ((Integer)(vWall.get( j ))).intValue() ].getY(), this.brickArray[ ((Integer)(vWall.get( j ))).intValue() ].getWidth(), this.brickArray[ ((Integer)(vWall.get( j ))).intValue() ].getHeight() );
				}
				return true;
			}
		}
		return false;
	}
	
	@SuppressWarnings("unchecked")
	private boolean setBrickCheckCorner( int i, double xSpeed, double ySpeed )
	{
		Vector	vBrick	= new Vector();
		if ( ( this.brickArray == null ) || ( this.brickArray.length <= 0 ) )
			return true;
		for ( int j = 0; j < this.brickArray.length; j++ )
		{
			if ( ( this.brickArray[j].getbBrick() ) && ( this.brickArray[j].getbWallHit( this.sweetyArray[i].getX(), this.sweetyArray[i].getY(), this.sweetyArray[i].getWidth(), this.sweetyArray[i].getHeight() ) ) )
			{
				vBrick.add( j );
			}
		}
		if ( vBrick.size() != 0 )
		{
			Vector vString 	= new Vector();
			Vector vWall	= new Vector();
			vString 	= new Vector();
			vWall		= new Vector();
			for ( int j = 0; j < vBrick.size(); j++ )
			{
				if ( this.brickArray[((Integer)vBrick.get( j )).intValue()].getbBrick() )
				{
					String s;
					s	= this.brickArray[ ((Integer)vBrick.get( j )).intValue() ].getCornerHit( this.sweetyArray[i].getX(), this.sweetyArray[i].getY(), this.sweetyArray[i].getWidth(), this.sweetyArray[i].getHeight(), xSpeed, ySpeed );
					if ( ( !s.equals("") ) )
					{
						vString.add( s );
						vWall.add(((Integer)vBrick.get( j )).intValue());
					}
				}
			}
			if ( vString.size() != 0 )
			{
				for ( int j = 0; j < vString.size(); j++ )
				{
					this.sweetyArray[i].setHindrance( vString.get( j ).toString(), this.brickArray[ ((Integer)(vWall.get( j ))).intValue() ].getX(), this.brickArray[ ((Integer)(vWall.get( j ))).intValue() ].getY(), this.brickArray[ ((Integer)(vWall.get( j ))).intValue() ].getWidth(), this.brickArray[ ((Integer)(vWall.get( j ))).intValue() ].getHeight() );
				}
				return true;
			}
		}
		return false;
	}
	
	private void setBombCheck()
	{
		for ( int i = 0; i < this.bombArray.length; i++ )
		{
			if ( this.bombArray[i].getbVisible() == false )
			{
				long t		= TimeUnit.NANOSECONDS.toMillis( System.nanoTime()- this.time );
				double time	= ((t/1000)%60) * 10 + (( t/100)%10);
				time		= time / 10;
				if ( ( !this.bombArray[i].getbStart() ) && ( this.bombArray[i].getExplodeTime() - 1 <= time ) )
				{
					this.bombArray[i].setbStart( true );
					this.setTimeCheck( i );
				} else if ( ( !this.bombArray[i].getbExplode() ) && ( this.bombArray[i].getExplodeTime() <= time ) )
				{
					this.bombArray[i].setExplodeBomb();
					this.setExplosionCheck( i );
				}
			}
		}
	}
	
	private void setTimeCheck( int bomb )
	{
		for ( int i = 0; i < this.sweetyArray.length; i++ )
		{
			this.sweetyArray[i].setTimeCheck( this.bombArray[bomb].getX() + this.bombArray[bomb].getWidth() / 2, this.bombArray[bomb].getY() + this.bombArray[bomb].getHeight() / 2 );
		}
	}
	
	private void setExplosionCheck( int bomb )
	{
		for ( int i = 0; i < this.sweetyArray.length; i++ )
		{
			this.sweetyArray[i].setSpeedCalculation( this.bombArray[bomb].getX() + this.bombArray[bomb].getWidth() / 2, this.bombArray[bomb].getY() + this.bombArray[bomb].getHeight() / 2 );
		}
	}
	
	private boolean setOutCheck()
	{
		for ( int i = 0; i < this.sweetyArray.length; i++ )
		{
			if ( 	( this.sweetyArray[i].getX() < - this.sweetyArray[i].getWidth() - 20 ) || 
					( this.sweetyArray[i].getX() > this.iBackground.getWidth() + this.sweetyArray[i].getWidth() + 20 ) ||
					( this.sweetyArray[i].getY() > this.sweetyArray[i].getHeight() + this.iBackground.getHeight() + 20 ) )
			{
				this.setThreadStop( true );
				return true;
			}
		}
		return false;
	}
	
	private void setSpeedCheck()
	{
		for ( int i = 0; i < this.bombArray.length; i++ )
		{
			if ( ( !this.bombArray[i].getbVisible() ) && ( !this.bombArray[i].getbExplode() ) )
				return;
		}
		for ( int i = 0; i < this.sweetyArray.length; i++ )
		{
			if ( ( this.sweetyArray[i].getXSpeed() != 0 ) || ( this.sweetyArray[i].getYSpeed() != 0 ) )
				return;
		}
		boolean bFinish		= true;
		for ( int i = 0; i < this.sweetyArray.length; i++ )
		{
			if ( !isInFinish( i ) )
			{
				bFinish		= false;
				break;
			}
		}
		if ( !bFinish )
		{
			this.setThreadStop( true );
		} else
		{
			this.bWin	= true;
			this.bombGame.setbWin( this.bWin );
			this.setThreadStop( false );
		}
	}
	
	private boolean isInFinish( int j )
	{
		for ( int i = 0; i < this.finishArray.length; i++ )
		{
			if ( this.finishArray[i].getbFinishHit( this.sweetyArray[j].getX(), this.sweetyArray[j].getY(), this.sweetyArray[j].getWidth(), this.sweetyArray[j].getHeight() ) )
			{
				return true;
			}
		}
		return false;
	}
	
	public void setFPS( String fps )
	{
		this.fps	= fps;
	}
	
	public void mouseReleasedRight( int x, int y )
	{
		if ( this.bWin )
		{
			this.level++;
			if ( this.bL.getLevelCount() == this.level )
				this.level	= 0;
			this.bWin	= false;
			this.setThreadStop( true );
			this.setNewLevel( this.bL, this.level );
			this.bombGame.setNewBomb( this.bL, this.level );
			return;
		}
		if ( !this.bThread )
		{
			for (int i = 0; i < this.bombArray.length; i++)
			{
				if (	(bombArray[i].getX() <= x) && (bombArray[i].getX() + bombArray[i].getWidth() > x) && 
						(bombArray[i].getY() <= y) && (bombArray[i].getY() + bombArray[i].getHeight() > y) && 
						(!bombArray[i].getbVisible()))
				{
					bombArray[i].setHud();
					break;
				}
			}
		} else if ( this.bThread )
		{
			this.bWin	= false;
			this.setThreadStop( true );
			return;
		}
	}
	
	public void mouseReleased( int x, int y )
	{
		if ( this.bWin )
		{
			this.level++;
			if ( this.bL.getLevelCount() == this.level )
				this.level	= 0;
			this.bWin	= false;
			this.setThreadStop( true );
			this.setNewLevel( this.bL, this.level );
			this.bombGame.setNewBomb( this.bL, this.level );
			return;
		}
		if ( !this.bThread )
		{
			for (int i = 0; i < this.bombArray.length; i++)
			{
				if (	(bombArray[i].getbPressed() == true) && (bombArray[i].getX() <= x) &&
						(bombArray[i].getX() + bombArray[i].getWidth() > x) && (bombArray[i].getY() <= y) &&
						(bombArray[i].getY() + bombArray[i].getHeight() > y) && (bombArray[i].getbVisible() == false))
				{
					bombArray[i].setbPressed( false );
				} else
				{
					bombArray[i].setbPressed( false );
					bombArray[i].setbSelected( false );
				}
			}
		}
	}
	
	public void mousePressed( int x, int y )
	{
		if ( this.bWin )
		{
			return;
		}
		if ( !this.bThread )
		{
			for (int i = 0; i < this.bombArray.length; i++)
			{
				if (	(!bombArray[i].getbVisible()) && (bombArray[i].getbOver()) && (!bombArray[i].getbSelected()) && (bombArray[i].getX() <= x) &&
						(bombArray[i].getX() + bombArray[i].getWidth() > x) && (bombArray[i].getY() <= y) &&
						(bombArray[i].getY() + bombArray[i].getHeight() > y))
				{
					for (int j = 0; j < this.bombArray.length; j++)
					{
						bombArray[j].setbPressed( false );
						bombArray[j].setbSelected( false );
					}
					bombArray[i].setbPressed( true );
					bombArray[i].setbSelected( true );
					bombArray[i].setXDiff( bombArray[i].getX() - x );
					bombArray[i].setYDiff( bombArray[i].getY() - y );
					this.bombGame.setBombSelect( i + 1 );
					break;
				}
			}
		}
	}
	
	public void mouseMove( int x, int y )
	{
		if ( this.bWin )
		{
			return;
		}
		if ( ( !this.bThread ) && ( this.bombArray != null ) )
		{
			for (int i = 0; i < this.bombArray.length; i++)
			{
				if (	(!bombArray[i].getbVisible()) && (!bombArray[i].getbOver()) && (bombArray[i].getX() <= x) &&
						(bombArray[i].getX() + bombArray[i].getWidth() > x) && (bombArray[i].getY() <= y) &&
						(bombArray[i].getY() + bombArray[i].getHeight() > y))
				{
					bombArray[i].setbOver( true );
					break;
				}
			}
		}
	}
	
	public void mouseDragged( int x, int y )
	{
		if ( this.bWin )
		{
			return;
		}
		if ( !this.bThread )
		{
			for (int i = 0; i < this.bombArray.length; i++)
			{
				if ( 	( bombArray[i].getbSelected() ) && ( ( bombArray[i].getX() + ( x - bombArray[i].getX() + bombArray[i].getXDiff() ) ) > 0 - this.bombArray[i].getWidth()/2 ) &&
						( ( bombArray[i].getX() + ( x - bombArray[i].getX() + bombArray[i].getXDiff() ) ) < 640 - bombArray[i].getWidth()/2 ) &&
						( ( bombArray[i].getY() + ( y - bombArray[i].getY() + bombArray[i].getYDiff() ) ) > 0 - bombArray[i].getHeight()/2 ) && 
						( ( bombArray[i].getY() + ( y - bombArray[i].getY() + bombArray[i].getYDiff() ) ) < 400 - bombArray[i].getHeight()/2 ) )
				{
					bombArray[i].setX( bombArray[i].getX() + ( x - bombArray[i].getX() + bombArray[i].getXDiff() ) );
					bombArray[i].setY( bombArray[i].getY() + ( y - bombArray[i].getY() + bombArray[i].getYDiff() ) );
				}
			}
		}
	}
	
	public void keyReleased( int key )
	{
		if ( !this.bThread && !this.bWin )
		{
			if ( key == 10 )
            {
				this.bombGame.setKeyStart( true );
                this.setThreadStart();
            } else if ( ( key >= 37 ) && ( key <= 40 ) )
            {
            	for (int i = 0; i < this.bombArray.length; i++)
                {
            		if ( 	( !this.bombArray[i].getbVisible() ) && ( this.bombArray[i].getbSelected() ) && ( this.bombArray[i].getX() > 0 - this.bombArray[i].getWidth()/2 ) &&
    						( this.bombArray[i].getX() < 640 - this.bombArray[i].getWidth()/2 ) &&
    						( this.bombArray[i].getY() > 0 - this.bombArray[i].getHeight()/2 ) && 
    						( this.bombArray[i].getY() < 400 - this.bombArray[i].getHeight()/2 ) )
            		{
            			if ( ( key == 37))
                        {
            				this.bombArray[i].setX( this.bombArray[i].getX() - 1 );
                        } else if ( ( key == 39))
                        {
                        	this.bombArray[i].setX( this.bombArray[i].getX() + 1 );
                        } else if ( ( key == 38))
                        {
                        	this.bombArray[i].setY( this.bombArray[i].getY() - 1 );
                        } else if ( ( key == 40))
                        {
                        	this.bombArray[i].setY( this.bombArray[i].getY() + 1 );
                        }
            			break;
            		}
                }
            } else if ( ( key >= 49 ) && ( key < this.bombArray.length + 49 ) )
            {
            	if ( this.bombArray[key-49].getbVisible() )
            	{
            		this.bombArray[key-49].setbPressed( false );
            		this.bombArray[key-49].setbVisible( false );
            	} else
            		this.bombArray[key-49].setHud();
            }
		} else if ( this.bThread )
		{
			if ( key == 10 )
            {
				this.bombGame.setKeyStart( false );
                this.setThreadStop( true );
            }
		} else if ( this.bWin )
		{
			if ( key == 10 )
            {
				this.level++;
				if ( this.bL.getLevelCount() == this.level )
					this.level	= 0;
				this.bWin	= false;
				this.setThreadStop( true );
				this.setNewLevel( this.bL, this.level );
				this.bombGame.setNewBomb( this.bL, this.level );
            }
		}
	}
	
	public void render( Graphics2D gfx ) 
	{
		gfx.drawImage( this.iBackground, 0, 0, null );
		
		if ( this.sweetyArray != null )
			for ( int i = 0; i < this.sweetyArray.length; i++ )
				this.sweetyArray[i].render( gfx );
		
		if ( this.wallArray != null )
			for ( int i = 0; i < this.wallArray.length; i++ )
				this.wallArray[i].render( gfx );
		
		if ( this.brickArray != null )
			for ( int i = 0; i < this.brickArray.length; i++ )
				this.brickArray[i].render( gfx );
		
		if ( this.bombArray != null )
			for ( int i = 0; i < this.bombArray.length; i++ )
				this.bombArray[i].render( gfx, false );
		
		if ( this.finishArray != null )
			for ( int i = 0; i < this.finishArray.length; i++ )
				this.finishArray[i].render( gfx );
		
		if ( !this.fps.equals( "" ) )
			gfx.drawString( this.fps, 280, 50 );
		
		long time	= 0;
		if ( this.time != 0 )
			time	= TimeUnit.NANOSECONDS.toMillis( System.nanoTime()- this.time );
		
		String min = ""+( time/1000/60);
		String sec = ""+(( time/1000)%60);
		String msec = ""+(( time/10)%100);
		if (sec.length()<2) sec = "0"+sec;
		if (msec.length()<2) msec = "0"+msec;
		String timeString = min+":"+sec+":"+msec;
		
		gfx.drawImage( this.iTimeImage, 220, 5, null );
		gfx.setFont( this.FontMiddle );
		gfx.setColor( Color.WHITE );
		gfx.drawString( timeString, 235, 26 );
		
		gfx.drawImage( this.iLevel, 330,  5, null );
		int x	= 0;
		if ( this.level >= 9 )
			x 	-= 10;
		gfx.drawString( ""+(this.level+1), 400 + x, 28 );
		
		if ( this.bWin == true )
		{
			gfx.drawImage( this.iWin, 20, 0, null );
		}
	}

}