package bomb;

import java.awt.Color;
import java.awt.Font;
import java.awt.Graphics2D;
import java.awt.image.BufferedImage;
import java.io.File;

public class BombGameHud
{
	private static final long serialVersionUID = 1L;

	private BombGame				bombGame;
	
	private BombImage				bombImage;
	
	private BufferedImage			iBackground, iBomb, iBombBackground, iPointBackground,
									iDragBackground, iDrag, iLevel;
	
	private BombButton[]			Buttons;
	private BombBombe[]				bombArray;
	private BombGameHudDrag			bombGameHudDrag;
	
	private int						bombSelect, level;
	
	private Font					bombFont;
	
	private boolean 				bThread, bWin;
	
	private File					fileSoundStart, fileSoundExplode;
	
	private BombLevel				bombLevel;
	
	public BombGameHud( BombGame bombGame, BombImage bombImage )
	{
		super();
		
		this.bombGame			= bombGame;
		this.bombImage			= bombImage;
	
		this.iBackground		= this.bombImage.setPicsMain( "/images/game_background_hud.png", false );
		this.iBomb				= this.bombImage.setPicsMain( "/images/bombe.png", false );
		this.iBombBackground	= this.bombImage.setPicsMain( "/images/game_hud_bombplace.png", false );
		this.iPointBackground	= this.bombImage.setPicsMain( "/images/game_hud_point_background.png", false );
		this.iLevel				= this.bombImage.setPicsMain( "/images/game_hud_level.png", false );
		this.iDragBackground	= this.bombImage.setPicsMain( "/images/game_hud_drag_background.png", false );
		this.iDrag				= this.bombImage.setPicsMain( "/images/game_hud_drag.png", false );
		
		this.bombGameHudDrag	= new BombGameHudDrag( this.iDragBackground, this.iDrag, 354, 460 );
		
		this.bombSelect			= 1;
		this.bombFont			= new Font( "Dialog", Font.BOLD, 29 );
		
		this.Buttons			= new BombButton[10];
		this.Buttons[0]			= new BombButton( this.bombImage.setPicsMain( "/images/game_hud_button_menu.png", false ), 580, 410, 50, 60, "Menu" );
		this.Buttons[1]			= new BombButton( this.bombImage.setPicsMain( "/images/game_hud_button_play.png", false ),  10, 410, 50, 60, "Play" );
		this.Buttons[2]			= new BombButton( this.bombImage.setPicsMain( "/images/game_hud_button_stop.png", false ),  10, 410, 50, 60, "Stop" );
		this.Buttons[2].setbVisible( false );
		this.Buttons[3]			= new BombButton( this.bombImage.setPicsMain( "/images/game_hud_button_left.png", false ),  360, 407, 20, 20, "Left" );
		this.Buttons[4]			= new BombButton( this.bombImage.setPicsMain( "/images/game_hud_button_right.png", false ), 450, 407, 20, 20, "Right" );
		this.Buttons[5]			= new BombButton( this.bombImage.setPicsMain( "/images/game_hud_button_left.png", false ),  360, 435, 20, 20, "LeftTime" );
		this.Buttons[6]			= new BombButton( this.bombImage.setPicsMain( "/images/game_hud_button_right.png", false ), 450, 435, 20, 20, "RightTime" );
		this.Buttons[7]			= new BombButton( this.bombImage.setPicsMain( "/images/game_hud_button_load.png", false ),  520, 445, 50, 25, "Load" );
		this.Buttons[8]			= new BombButton( this.bombImage.setPicsMain( "/images/game_hud_button_left.png", false ),  520, 421, 20, 20, "LeftLevel" );
		this.Buttons[9]			= new BombButton( this.bombImage.setPicsMain( "/images/game_hud_button_right.png", false ), 550, 421, 20, 20, "RightLevel" );
		
		this.fileSoundStart		= new File( System.getProperty("user.dir")+"/sounds/Hiss.wav");
		this.fileSoundExplode	= new File( System.getProperty("user.dir")+"/sounds/Explode.wav"); 
		
		this.bombArray			= new BombBombe[5];
		int x					= 80;
		int y					= 24;
		for ( int i = 0; i < 5; i++ )
		{
			this.bombArray[i]	= new BombBombe( this.iBomb, this.iBombBackground, x, y, this.fileSoundStart, this.fileSoundExplode );
			x					+= 55;
		}
		
		this.level				= 0;
        
        this.bThread			= false;
        this.bWin				= false;
	}
	
	public void setNewLevel( BombLevel bombLevel, int level )
	{
		this.bombLevel			= bombLevel;
		this.level				= level;
		//System.out.println("level = "+this.level+" bombCount = "+bL.getBombCount( this.level ) );
		this.bombSelect			= 1;
		this.bombArray			= new BombBombe[bombLevel.getBombCount( this.level )];
		int x					= 80;
		int y					= 24;
		for ( int i = 0; i < this.bombArray.length; i++ )
		{
			this.bombArray[i]	= new BombBombe( this.iBomb, this.iBombBackground, x, y, this.fileSoundStart, this.fileSoundExplode );
			x					+= 55;
		}
	}
	
	public BombBombe[]	getBombArray()
	{
		return this.bombArray;
	}
	
	public void setbWin( boolean bWin )
	{
		this.bWin			= bWin;
	}
	
	public void setbThread( boolean bThread )
	{
		this.bThread		= bThread;
		if (( this.Buttons[2].getbVisible() ) && ( !this.bThread ) )
		{
			this.Buttons[1].setbOver( true );
			this.Buttons[1].setbVisible( true );
			this.Buttons[2].setbOver( false );
			this.Buttons[2].setbVisible( false );
		}
	}
	
	public void setBombSelect( int bombSelect )
	{
		this.bombSelect	= bombSelect;
		this.bombGameHudDrag.setXDiff( (int)(this.bombArray[ this.bombSelect - 1 ].getExplodeTime() * 10), false );
	}
	
	public void setStart( boolean bStart )
	{
		if ( bStart )
		{
			this.Buttons[1].setbVisible( false );
			this.Buttons[2].setbVisible( true );
		} else
		{
			this.Buttons[1].setbVisible( true );
			this.Buttons[2].setbVisible( false );
		}
	}
	
	public void think()
	{
		
	}
	
	public void mouseReleased( int x, int y )
	{
		boolean bReady	= false;
		for (int i = 0; i < Buttons.length; i++)
		{
			if ( (Buttons[i].getbPressed()) && (Buttons[i].getx() <= x) &&
				 (Buttons[i].getx() + Buttons[i].getWidth() > x) && (Buttons[i].gety() <= y) &&
				 (Buttons[i].gety() + Buttons[i].getHeight() > y) && (Buttons[i].getbVisible()))
			{
				Buttons[i].setbPressed( false );
				if ( this.bWin )
				{
					return;
				}
				if ("Menu".equals( Buttons[i].getFunction() ))
				{
					if ( !this.bThread )
					{
						this.bombGame.setMenu();
						return;
					}
				}
				else if ("Load".equals( Buttons[i].getFunction() ))
				{
					if ( !this.bThread )
						this.bombGame.load();
				}
				else if ("Play".equals( Buttons[i].getFunction() ))
				{
					this.Buttons[i].setbOver( true );
					this.Buttons[i].setbVisible( false );
					this.Buttons[i+1].setbOver( true );
					this.Buttons[i+1].setbVisible( true );
					this.bombGame.setThreadStart();
				}
				else if ("Stop".equals( Buttons[i].getFunction() ))
				{
					this.Buttons[i-1].setbOver( true );
					this.Buttons[i-1].setbVisible( true );
					this.Buttons[i].setbOver( true );
					this.Buttons[i].setbVisible( false );
					this.bombGame.setThreadStop();
				}
				else if ("Left".equals( Buttons[i].getFunction() ))
				{
					if ( !this.bThread )
					{
						for ( int j = 0; j < this.bombArray.length; j++ )
						{
							this.bombArray[j].setbSelected( false );
						}
						this.bombSelect--;
						if ( this.bombSelect <= 0 )
							this.bombSelect	= this.bombArray.length;
						this.bombArray[ this.bombSelect - 1 ].setbSelected( true );
						bReady	= true;
						this.bombGameHudDrag.setXDiff( (int)(this.bombArray[ this.bombSelect - 1 ].getExplodeTime() * 10), false );
						break;
					}
				}
				else if ("Right".equals( Buttons[i].getFunction() ))
				{
					if ( !this.bThread )
					{
						for ( int j = 0; j < this.bombArray.length; j++ )
						{
							this.bombArray[j].setbSelected( false );
						}
						this.bombSelect++;
						if ( this.bombSelect > this.bombArray.length )
							this.bombSelect	= 1;
						this.bombArray[ this.bombSelect - 1 ].setbSelected( true );
						bReady	= true;
						this.bombGameHudDrag.setXDiff( (int)(this.bombArray[ this.bombSelect - 1 ].getExplodeTime() * 10), false );
						break;
					}
				}
				else if ("LeftTime".equals( Buttons[i].getFunction() ))
				{
					if ( !this.bThread )
					{
						this.bombArray[ this.bombSelect - 1 ].setExplodeTime( this.bombArray[ this.bombSelect - 1 ].getExplodeTime() - 0.1D );
						this.bombGameHudDrag.setXDiff( -1, true );
					}
				}
				else if ("RightTime".equals( Buttons[i].getFunction() ))
				{
					if ( !this.bThread )
					{
						this.bombArray[ this.bombSelect - 1 ].setExplodeTime( this.bombArray[ this.bombSelect - 1 ].getExplodeTime() + 0.1D );
						this.bombGameHudDrag.setXDiff( 1, true );
					}
				}
				else if ("LeftLevel".equals( Buttons[i].getFunction() ))
				{
					this.level--;
					if ( this.level < 0 )
						this.level	= this.bombLevel.getLevelCount()-1;
					this.bombGame.setLoadLevel( this.bombLevel, this.level );
				}
				else if ("RightLevel".equals( Buttons[i].getFunction() ))
				{
					this.level++;
					if ( this.level >= this.bombLevel.getLevelCount() )
						this.level	= 0;
					this.bombGame.setLoadLevel( this.bombLevel, this.level );
				}
				bReady	= true;
				break;
			} else
			{
				Buttons[i].setbOver( false );
				Buttons[i].setbPressed( false );
			}
		}
		if ( this.bWin )
		{
			return;
		}
		if ( !bReady )
		{
			//System.out.println("Bei Bomb!");
			for (int i = 0; i < this.bombArray.length; i++)
			{
				if (	(!this.bThread) && (this.bombArray[i].getbPressed()) && (this.bombArray[i].getXBackground() <= x) &&
						(this.bombArray[i].getXBackground() + this.bombArray[i].getWidth() > x) && (this.bombArray[i].getYBackground() <= y - 400 ) &&
						(this.bombArray[i].getYBackground() + this.bombArray[i].getHeight() > y - 400 ) && (this.bombArray[i].getbVisible()))
				{
					//System.out.println("Bombe Nummer "+(i+1)+" wird ins Spiel gesetzt!");
					this.bombArray[i].setbPressed( false );
					this.bombArray[i].setbVisible( false );
					bReady =	true;
					break;
				}
			}
		}
		if ( !bReady )
		{
			if (	(!this.bThread) && (this.bombGameHudDrag.getbPressed()) && (this.bombGameHudDrag.getX() <= x) &&
					(this.bombGameHudDrag.getX() + this.bombGameHudDrag.getWidth() > x) && (this.bombGameHudDrag.getY() <= y) &&
					(this.bombGameHudDrag.getY() + this.bombGameHudDrag.getHeight() > y))
			{
				this.bombGameHudDrag.setbPressed( false );
			} else
			{
				this.bombGameHudDrag.setbOver( false );
				this.bombGameHudDrag.setbPressed( false );
			}
		}
	}
	
	public void mousePressed( int x, int y )
	{
		boolean bReady	= false;
		for (int i = 0; i < Buttons.length; i++)
		{
			if ( (Buttons[i].getbVisible()) && (Buttons[i].getbOver()) && (Buttons[i].getx() <= x) &&
				 (Buttons[i].getx() + Buttons[i].getWidth() > x) && (Buttons[i].gety() <= y) &&
				 (Buttons[i].gety() + Buttons[i].getHeight() > y))
			{
				Buttons[i].setbPressed( true );
				bReady	= true;
				break;
			}
		}
		if ( !bReady )
		{
			for (int i = 0; i < this.bombArray.length; i++)
			{
				if (	(!this.bThread) && (bombArray[i].getbVisible()) && (bombArray[i].getbOver()) && (bombArray[i].getXBackground() <= x) &&
						(bombArray[i].getXBackground() + bombArray[i].getWidth() > x) && (bombArray[i].getYBackground() <= y - 400 ) &&
						(bombArray[i].getYBackground() + bombArray[i].getHeight() > y - 400 ))
				{
					bombArray[i].setbPressed( true );
					bReady	= true;
					break;
				}
			}
		}
		if ( !bReady )
		{
			if (	( !this.bWin ) && (!this.bThread) && (this.bombGameHudDrag.getbOver()) && (this.bombGameHudDrag.getX() <= x) &&
					(this.bombGameHudDrag.getX() + this.bombGameHudDrag.getWidth() > x) && (this.bombGameHudDrag.getY() <= y) &&
					(this.bombGameHudDrag.getY() + this.bombGameHudDrag.getHeight() > y))
			{
				this.bombGameHudDrag.setbPressed( true );
				this.bombGameHudDrag.setXMouse( x );
			} else
			{
				this.bombGameHudDrag.setbOver( false );
				this.bombGameHudDrag.setbPressed( false );
			}
		}
	}
	
	public void mouseMove( int x, int y )
	{
		boolean bReady	= false;
		for (int i = 0; i < Buttons.length; i++)
		{
			if ( (Buttons[i].getbVisible()) && (!Buttons[i].getbOver()) && (Buttons[i].getx() <= x) &&
				 (Buttons[i].getx() + Buttons[i].getWidth() > x) && (Buttons[i].gety() <= y) &&
				 (Buttons[i].gety() + Buttons[i].getHeight() > y))
			{
				Buttons[i].setbOver( true );
				bReady	= true;
			} else if (	(Buttons[i].getbOver()) && ((Buttons[i].getx() > x) ||
					 	(Buttons[i].getx() + Buttons[i].getWidth() <= x) || (Buttons[i].gety() > y) ||
					 	(Buttons[i].gety() + Buttons[i].getHeight() <= y)) )
			{
				Buttons[i].setbOver( false );
				Buttons[i].setbPressed( false );
			}
		}
		if ( bReady == false )
		{
			for (int i = 0; i < this.bombArray.length; i++)
			{
				if (	( !this.bThread) && (bombArray[i].getbVisible()) && (!bombArray[i].getbOver()) && (bombArray[i].getXBackground() <= x) &&
						(bombArray[i].getXBackground() + bombArray[i].getWidth() > x) && (bombArray[i].getYBackground() <= y - 400 ) &&
						(bombArray[i].getYBackground() + bombArray[i].getHeight() > y - 400 ))
				{
					bombArray[i].setbOver( true );
					bReady		= true;
					break;
				}
			}
		}
		if ( bReady == false )
		{
			if (	(!this.bThread) && (!this.bombGameHudDrag.getbOver()) && (this.bombGameHudDrag.getX() <= x) &&
					(this.bombGameHudDrag.getX() + this.bombGameHudDrag.getWidth() > x) && (this.bombGameHudDrag.getY() <= y) &&
					(this.bombGameHudDrag.getY() + this.bombGameHudDrag.getHeight() > y))
			{
				this.bombGameHudDrag.setbOver( true );
			} else if (	(this.bombGameHudDrag.getbOver()) && ((this.bombGameHudDrag.getX() > x) ||
				 		(this.bombGameHudDrag.getX() + this.bombGameHudDrag.getWidth() <= x) || (this.bombGameHudDrag.getY() > y) ||
				 		(this.bombGameHudDrag.getY() + this.bombGameHudDrag.getHeight() <= y)) )
			{
				this.bombGameHudDrag.setbOver( false );
				this.bombGameHudDrag.setbPressed( false );
			}
		}
	}
	
	public void mouseDragged( int x )
	{
		if (	(!this.bThread) && (this.bombGameHudDrag.getbPressed()))
		{
			int xDiffOld	= this.bombGameHudDrag.getXDiff();
			this.bombGameHudDrag.setXDiff( x );
			double	explodeChange	= (double)(- xDiffOld + this.bombGameHudDrag.getXDiff());
			this.bombArray[ this.bombSelect - 1 ].setExplodeTime(  this.bombArray[ this.bombSelect - 1 ].getExplodeTime() + (explodeChange / 10) );
		}
	}
	
	public void render(Graphics2D gfx ) 
	{
		gfx.drawImage( this.iBackground, 0, 400, null );
		gfx.drawImage( this.iLevel, 519, 404, null );
		for ( int i = 0; i < this.Buttons.length; i++ )
		{
			this.Buttons[i].render( gfx );
		}
		for ( int i = 0; i < this.bombArray.length; i++ )
		{
			this.bombArray[i].render( gfx, true );
		}
		gfx.drawImage( this.iPointBackground, 390,  404, null );
		gfx.drawImage( this.iBomb, 403, 405, 427, 429, 0, 0, this.iBomb.getWidth(), this.iBomb.getHeight(), null );
		gfx.drawImage( this.iPointBackground, 390, 432, null );
		this.bombGameHudDrag.render( gfx );
		gfx.setFont( this.bombFont );
		gfx.setColor( Color.WHITE );
		gfx.drawString( ""+this.bombSelect, 410, 428 );
		gfx.drawString( ""+this.bombArray[ this.bombSelect - 1 ].getExplodeTime(), 395, 456 );
	}

}
