package bomb;

import java.awt.image.BufferedImage;
import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.EOFException;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.Vector;

public class BombLevel
{
	private BombWall[][]	aWall;
	private BombSweety[][]	aSweety;
	private BombFinish[][]	aFinish;
	private BombBrick[][]	aBrick;
	private int 			levelCount;
	private int[]			bombCount, wallCount, sweetyCount, finishCount, brickCount;
	private BufferedImage	iWall, iBrick, iBrickExplode, iSweetyNormal, iSweetyTime, iSweetyExplode;
	private String[]		aName;
	private String			fileName;
	
	public BombLevel( BufferedImage iWall, BufferedImage iBrick, BufferedImage iBrickExplode, BufferedImage iSweetyNormal, BufferedImage iSweetyTime, BufferedImage iSweetyExplode )
	{
		super();
		
		this.iWall			= iWall;
		this.iBrick			= iBrick;
		this.iBrickExplode	= iBrickExplode;
		this.iSweetyNormal	= iSweetyNormal;
		this.iSweetyTime	= iSweetyTime;
		this.iSweetyExplode	= iSweetyExplode;
		
		this.levelCount		= 0;
	}
	
	public void changeLevel( int k, int newK )
	{
		BombWall[] tempWall		= this.aWall[newK];
		this.aWall[newK]		= this.aWall[k];
		this.aWall[k]			= tempWall;
		BombBrick[] tempBrick	= this.aBrick[newK];
		this.aBrick[newK]		= this.aBrick[k];
		this.aBrick[k]			= tempBrick;
		BombSweety[] tempSweety	= this.aSweety[newK];
		this.aSweety[newK]		= this.aSweety[k];
		this.aSweety[k]			= tempSweety;
		BombFinish[] tempFinish	= this.aFinish[newK];
		this.aFinish[newK]		= this.aFinish[k];
		this.aFinish[k]			= tempFinish;
		
		int tempBombCount		= this.bombCount[newK];
		this.bombCount[newK]	= this.bombCount[k];
		this.bombCount[k]		= tempBombCount;
		int tempWallCount		= this.wallCount[newK];
		this.wallCount[newK]	= this.wallCount[k];
		this.wallCount[k]		= tempWallCount;
		int tempBrickCount		= this.brickCount[newK];
		this.brickCount[newK]	= this.brickCount[k];
		this.brickCount[k]		= tempBrickCount;
		int tempSweetyCount		= this.sweetyCount[newK];
		this.sweetyCount[newK]	= this.sweetyCount[k];
		this.sweetyCount[k]		= tempSweetyCount;
		int tempFinishCount		= this.finishCount[newK];
		this.finishCount[newK]	= this.finishCount[k];
		this.finishCount[k]		= tempFinishCount;
		String tempAName		= this.aName[newK];
		this.aName[newK]		= this.aName[k];
		this.aName[k]			= tempAName;
	}
	
	public void setAll( Vector<BombWall> vWall, Vector<BombBrick> vBrick, Vector<BombSweety> vSweety, Vector<BombFinish> vFinish, int bombCount, String name, int k )
	{
		//System.out.println("k = "+k);
		BombWall[] tempWall	= new BombWall[vWall.size()];
		for ( int i = 0; i < vWall.size(); i++ )
		{
			tempWall[i]		= (vWall.get(i));
		}
		this.aWall[k]		= tempWall;
		this.wallCount[k]	= vWall.size();
		BombBrick[] tempBrick	= new BombBrick[vBrick.size()];
		for ( int i = 0; i < vBrick.size(); i++ )
		{
			tempBrick[i]		= (vBrick.get(i));
		}
		this.aBrick[k]		= tempBrick;
		this.brickCount[k]	= vBrick.size();
		BombSweety[] tempSweety	= new BombSweety[vSweety.size()];
		for ( int i = 0; i < vSweety.size(); i++ )
		{
			tempSweety[i]		= (vSweety.get(i));
		}
		this.aSweety[k]		= tempSweety;
		this.sweetyCount[k]	= vSweety.size();
		BombFinish[] tempFinish	= new BombFinish[vFinish.size()];
		for ( int i = 0; i < vFinish.size(); i++ )
		{
			tempFinish[i]		= (vFinish.get(i));
		}
		this.aFinish[k]		= tempFinish;
		this.finishCount[k]	= vFinish.size();
		
		this.bombCount[k]	= bombCount;
		this.aName[k]		= name;
	}
	
	public void setEmptyLevel()
	{
		this.bombCount		= null;
		this.wallCount		= null;
		this.brickCount		= null;
		this.sweetyCount	= null;
		this.finishCount	= null;
		this.aName			= null;
		this.aWall			= null;
		this.aBrick			= null;
		this.aSweety		= null;
		this.aFinish		= null;
		this.levelCount		= 0;
		this.setNewLevel();
	}
	
	public void setNewLevel()
	{
		this.levelCount++;
		int[] newBombCount	= new int[ this.levelCount ];
		if ( this.bombCount != null )
			System.arraycopy(this.bombCount,0,newBombCount,0,this.levelCount-1);
		newBombCount[this.levelCount-1]	= 5;
		this.bombCount		= newBombCount;
		int[] newWallCount	= new int[ this.levelCount ];
		if ( this.wallCount != null )
			System.arraycopy(this.wallCount,0,newWallCount,0,this.levelCount-1);
		newWallCount[this.levelCount-1]	= 0;
		this.wallCount		= newWallCount;
		int[] newBrickCount	= new int[ this.levelCount ];
		if ( this.brickCount != null )
			System.arraycopy(this.brickCount,0,newBrickCount,0,this.levelCount-1);
		newBrickCount[this.levelCount-1]	= 0;
		this.brickCount		= newBrickCount;
		int[] newSweetyCount	= new int[ this.levelCount ];
		if ( this.sweetyCount != null )
			System.arraycopy(this.sweetyCount,0,newSweetyCount,0,this.levelCount-1);
		newSweetyCount[this.levelCount-1]	= 0;
		this.sweetyCount		= newSweetyCount;
		int[] newFinishCount	= new int[ this.levelCount ];
		if ( this.finishCount != null )
			System.arraycopy(this.finishCount,0,newFinishCount,0,this.levelCount-1);
		newFinishCount[this.levelCount-1]	= 0;
		this.finishCount		= newFinishCount;
		String[] newName	= new String[ this.levelCount ];
		if ( this.aName != null )
			System.arraycopy(this.aName,0,newName,0,this.levelCount-1);
		newName[this.levelCount-1]	= "Apo";
		this.aName		= newName;
		
		BombWall[][] newBombWall	= new BombWall[this.levelCount][1000];
		if ( this.aWall != null )
		{
			for ( int i = 0; i < this.levelCount-1; i++ )
			{
				newBombWall[i]	= this.aWall[i];
			}
		}
		this.aWall			= newBombWall;
		BombBrick[][] newBombBrick	= new BombBrick[this.levelCount][1000];
		if ( this.aBrick != null )
		{
			for ( int i = 0; i < this.levelCount-1; i++ )
			{
				newBombBrick[i]	= this.aBrick[i];
			}
		}
		this.aBrick			= newBombBrick;
		BombSweety[][] newBombSweety	= new BombSweety[this.levelCount][1000];
		if ( this.aSweety != null )
		{
			for ( int i = 0; i < this.levelCount-1; i++ )
			{
				newBombSweety[i]	= this.aSweety[i];
			}
		}
		this.aSweety			= newBombSweety;
		BombFinish[][] newBombFinish	= new BombFinish[this.levelCount][1000];
		if ( this.aFinish != null )
		{
			for ( int i = 0; i < this.levelCount-1; i++ )
			{
				newBombFinish[i]	= this.aFinish[i];
			}
		}
		this.aFinish			= newBombFinish;
	}
	
	public void setDelete( int level )
	{
		this.levelCount--;
		System.out.println("level = "+level+" mit levelCount = "+this.levelCount);
		int count			= 0;
		int[] newBombCount	= new int[ this.levelCount ];
		for ( int i = 0; i < this.levelCount+1; i++ )
		{
			if ( level == i )
			{
				count++;
			} else
			{
				//System.out.println(i+" "+this.bombCount[i]);
				newBombCount[i-count]	= this.bombCount[i];
			}
		}
		this.bombCount		= newBombCount;
		count			= 0;
		int[] newWallCount	= new int[ this.levelCount ];
		for ( int i = 0; i < this.levelCount+1; i++ )
		{
			if ( level == i )
				count++;
			else
				newWallCount[i-count]	= this.wallCount[i];
		}
		this.wallCount		= newWallCount;
		
		count			= 0;
		int[] newBrickCount	= new int[ this.levelCount ];
		for ( int i = 0; i < this.levelCount+1; i++ )
		{
			if ( level == i )
				count++;
			else
				newBrickCount[i-count]	= this.brickCount[i];
		}
		this.brickCount		= newBrickCount;
		
		count			= 0;
		int[] newSweetyCount	= new int[ this.levelCount ];
		for ( int i = 0; i < this.levelCount+1; i++ )
		{
			if ( level == i )
				count++;
			else
				newSweetyCount[i-count]	= this.sweetyCount[i];
		}
		this.sweetyCount		= newSweetyCount;
		count			= 0;
		int[] newFinishCount	= new int[ this.levelCount ];
		for ( int i = 0; i < this.levelCount+1; i++ )
		{
			if ( level == i )
				count++;
			else
				newFinishCount[i-count]	= this.finishCount[i];
		}
		this.finishCount		= newFinishCount;
		count			= 0;
		String[] newName	= new String[ this.levelCount ];
		for ( int i = 0; i < this.levelCount+1; i++ )
		{
			if ( level == i )
				count++;
			else
				newName[i-count]	= this.aName[i];
		}
		this.aName				= newName;
		count			= 0;
		BombWall[][] newBombWall	= new BombWall[ this.levelCount ][1000];
		for ( int i = 0; i < this.levelCount+1; i++ )
		{
			if ( level == i )
				count++;
			else
				newBombWall[i-count]	= this.aWall[i];
		}
		this.aWall					= newBombWall;
		
		count			= 0;
		BombBrick[][] newBombBrick	= new BombBrick[ this.levelCount ][1000];
		for ( int i = 0; i < this.levelCount+1; i++ )
		{
			if ( level == i )
				count++;
			else
				newBombBrick[i-count]	= this.aBrick[i];
		}
		this.aBrick					= newBombBrick;
		
		count			= 0;
		BombSweety[][] newBombSweety	= new BombSweety[ this.levelCount ][1000];
		for ( int i = 0; i < this.levelCount+1; i++ )
		{
			if ( level == i )
				count++;
			else
				newBombSweety[i-count]	= this.aSweety[i];
		}
		this.aSweety				= newBombSweety;
		count			= 0;
		BombFinish[][] newBombFinish	= new BombFinish[ this.levelCount ][1000];
		for ( int i = 0; i < this.levelCount+1; i++ )
		{
			if ( level == i )
				count++;
			else
				newBombFinish[i-count]	= this.aFinish[i];
		}
		this.aFinish				= newBombFinish;
	}
	
	public String getFileName()
	{
		return this.fileName;
	}
	
	public String getName( int k )
	{
		return this.aName[k];
	}
	
	public void setName( String name, int k )
	{
		this.aName[k]	= name;
	}
	
	public int getLevelCount()
	{
		return this.levelCount;
	}
	
	public int getBombCount( int k )
	{
		if ( this.bombCount == null )
			return 5;
		return this.bombCount[k];
	}
	
	public BombWall[] getWall( int k )
	{
		if ( this.aWall == null )
			return null;
		int count	= 0;
		for ( int i = 0; i < this.aWall[k].length; i++ )
		{
			if ( this.aWall[k][i] != null )
				count++;
			else
				break;
		}
		BombWall[] aWall	= new BombWall[count];
		for ( int i = 0; i < count; i++ )
		{
			aWall[i]	= this.aWall[k][i];
		}
		return aWall;
	}
	
	@SuppressWarnings("unchecked")
	public Vector<BombWall> getWallVector( int k )
	{
		Vector<BombWall> vWall	= new Vector<BombWall>();
		if ( this.aWall[k] != null )
		{
			for ( int i = 0; i < this.aWall[k].length; i++ )
			{
				if ( this.aWall[k][i] == null )
					return vWall;
				vWall.add( this.aWall[k][i] );
			}
		}
		return vWall;
	}
	
	public BombBrick[] getBrick( int k )
	{
		if ( this.aBrick == null )
			return null;
		int count	= 0;
		for ( int i = 0; i < this.aBrick[k].length; i++ )
		{
			if ( this.aBrick[k][i] != null )
				count++;
			else
				break;
		}
		BombBrick[] aBrick	= new BombBrick[count];
		for ( int i = 0; i < count; i++ )
		{
			aBrick[i]	= this.aBrick[k][i];
		}
		return aBrick;
	}
	
	@SuppressWarnings("unchecked")
	public Vector<BombBrick> getBrickVector( int k )
	{
		Vector<BombBrick> vBrick	= new Vector<BombBrick>();
		if ( this.aBrick[k] != null )
		{
			for ( int i = 0; i < this.aBrick[k].length; i++ )
			{
				if ( this.aBrick[k][i] == null )
					return vBrick;
				vBrick.add( this.aBrick[k][i] );
			}
		}
		return vBrick;
	}
	
	public BombSweety[] getSweety( int k )
	{
		int count	= 0;
		for ( int i = 0; i < this.aSweety[k].length; i++ )
		{
			if ( this.aSweety[k][i] != null )
				count++;
			else
				break;
		}
		BombSweety[] aSweety	= new BombSweety[count];
		for ( int i = 0; i < count; i++ )
		{
			aSweety[i]	= this.aSweety[k][i];
		}
		return aSweety;
	}
	
	@SuppressWarnings("unchecked")
	public Vector<BombSweety> getSweetyVector( int k )
	{
		Vector<BombSweety> vSweety	= new Vector<BombSweety>();
		if ( this.aSweety[k] != null )
			for ( int i = 0; i < this.aSweety[k].length; i++ )
			{
				if ( this.aSweety[k][i] == null )
					return vSweety;
				vSweety.add( this.aSweety[k][i] );
			}
		return vSweety;
	}
	
	public BombFinish[] getFinish( int k )
	{
		if ( this.aFinish == null )
			return null;
		int count	= 0;
		for ( int i = 0; i < this.aFinish[k].length; i++ )
		{
			if ( this.aFinish[k][i] != null )
				count++;
			else
				break;
		}
		BombFinish[] aFinish	= new BombFinish[count];
		for ( int i = 0; i < count; i++ )
		{
			aFinish[i]	= this.aFinish[k][i];
		}
		return aFinish;
	}
	
	@SuppressWarnings("unchecked")
	public Vector<BombFinish> getFinishVector( int k )
	{
		Vector<BombFinish> vFinish	= new Vector<BombFinish>();
		if ( this.aFinish[k] != null )
			for ( int i = 0; i < this.aFinish[k].length; i++ )
			{
				if ( this.aFinish[k][i] == null )
					return vFinish;
				vFinish.add( this.aFinish[k][i] );
			}
		return vFinish;
	}
	
	/**
	 * schaut ob so eine Datei bereits existiert
	 * @param fileName = Angabe, wo die Datei liegt
	 * @return true, falls existiert, sonst false 
	 */
	protected boolean isExisting( String fileName )
	{
		try
		{
			FileInputStream file		= new FileInputStream( fileName );
			file.close();
			return true;
		} catch ( IOException e )
		{
			//System.out.println("Error: "+e);
			return false;
		}
	}
	
	/**
	 * schreibt eine Datei in das bergebene String Filesystem
	 * @param fileName = wo wird hingespeichert
	 */
	protected void writeLevel( String fileName )
	{
		try
		{
			FileOutputStream file		= new FileOutputStream( fileName );
			BufferedOutputStream buff	= new BufferedOutputStream( file );
			DataOutputStream data		= new DataOutputStream( buff );

			data.writeInt( this.levelCount );
			
			for ( int i = 0; i < this.levelCount; i++ )
			{
				data.writeInt( this.wallCount[i] );
				for ( int k = 0; k < this.wallCount[i]; k++ )
				{
					data.writeInt( this.aWall[i][k].getX() );
					data.writeInt( this.aWall[i][k].getY() );
				}
				data.writeInt( this.brickCount[i] );
				for ( int k = 0; k < this.brickCount[i]; k++ )
				{
					data.writeInt( this.aBrick[i][k].getX() );
					data.writeInt( this.aBrick[i][k].getY() );
				}
				data.writeInt( this.sweetyCount[i] );
				for ( int k = 0; k < this.sweetyCount[i]; k++ )
				{
					data.writeInt( (int)this.aSweety[i][k].getX() );
					data.writeInt( (int)this.aSweety[i][k].getY() );
				}
				data.writeInt( this.finishCount[i] );
				for ( int k = 0; k < this.finishCount[i]; k++ )
				{
					data.writeInt( (int)this.aFinish[i][k].getX() );
					data.writeInt( (int)this.aFinish[i][k].getY() );
					data.writeInt( this.aFinish[i][k].getHeight() );
					data.writeInt( this.aFinish[i][k].getWidth() );
				}
				data.writeInt( this.bombCount[i] );
				data.writeUTF( this.aName[i] );
			}
			data.close();
		} catch ( IOException e )
		{
			System.out.println("Error: "+e);
		}
	}
	
	/**
	 * liest eine Datei in das bergebene String Filesystem
	 * @param fileName = von wo wird gelesen
	 */
	protected void readLevel( String fileName )
	{
		try
		{
			FileInputStream file		= new FileInputStream( fileName );
			BufferedInputStream buff	= new BufferedInputStream( file );
			DataInputStream data		= new DataInputStream( buff );
			try
			{
				this.fileName		= fileName;
				this.levelCount		= data.readInt();
				this.bombCount		= new int[this.levelCount];
				this.wallCount		= new int[this.levelCount];
				this.brickCount		= new int[this.levelCount];
				this.sweetyCount	= new int[this.levelCount];
				this.finishCount	= new int[this.levelCount];
				this.aName			= new String[this.levelCount];
				//System.out.println("Check mit levelCount "+this.levelCount);
				for ( int i = 0; i < this.levelCount; i++ )
				{
					this.wallCount[i]	= data.readInt();
					if ( i == 0 )
						this.aWall		= new BombWall[this.levelCount][ 1000 ];
					for ( int k = 0; k < this.wallCount[i]; k++ )
					{
						int x	= data.readInt();
						int y	= data.readInt();
						this.aWall[i][k]	= new BombWall( x, y, this.iWall );
					}
					this.brickCount[i]	= data.readInt();
					//System.out.println(this.brickCount[i]);
					if ( i == 0 )
						this.aBrick		= new BombBrick[this.levelCount][ 1000 ];
					for ( int k = 0; k < this.brickCount[i]; k++ )
					{
						int x	= data.readInt();
						int y	= data.readInt();
						this.aBrick[i][k]	= new BombBrick( x, y, this.iBrick, this.iBrickExplode );
					}
					this.sweetyCount[i]	= data.readInt();
					if ( i == 0 )
						this.aSweety		= new BombSweety[this.levelCount][ 1000 ];
					for ( int k = 0; k < this.sweetyCount[i]; k++ )
					{
						int x	= data.readInt();
						int y	= data.readInt();
						this.aSweety[i][k]	= new BombSweety( x, y, this.iSweetyNormal.getWidth(), this.iSweetyNormal.getHeight(), this.iSweetyNormal, this.iSweetyTime, this.iSweetyExplode );
					}
					this.finishCount[i]	= data.readInt();
					if ( ( i == 0 ) || ( this.aFinish == null ) )
						this.aFinish	= new BombFinish[this.levelCount][ 1000 ];
					for ( int k = 0; k < this.finishCount[i]; k++ )
					{
						int x 		= data.readInt();
						int y 		= data.readInt();
						int height	= data.readInt();
						int width	= data.readInt();
						this.aFinish[i][k]	= new BombFinish( x, y, width, height );
					}
					this.bombCount[i]		= data.readInt();
					this.aName[i]			= data.readUTF();
				}
				data.close();
			} catch ( EOFException e )
			{
				System.out.println("Error: "+e);
				buff.close();
			}
			data.close();
		} catch ( IOException e )
		{
			System.out.println("Error: "+e);
		}
	}

}
