package bomb;

import java.awt.*;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.awt.image.BufferStrategy;
import java.io.File;
import java.util.Vector;

import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JPanel;

public class BombMain extends Canvas implements MouseListener, MouseMotionListener
{

	private static final long 			serialVersionUID = 1L;
	
	private final JFileChooser			fc = new JFileChooser();
	private final BombClassFileFilter	cff = new BombClassFileFilter();
	
	/** The stragey that allows us to use accelerate page flipping */
	private BufferStrategy strategy;
	/** True if the game is currently "running", i.e. the game loop is looping */
	private boolean bGameRunning = true;
	/** The current accelerated graphics context */
	private Graphics2D g;
	
	private BombGame					bombGame;
	private BombEditor					bombEditor;
	private BombMenu					bombMenu;
	
	private BombImage					bombImage;
	
	private static BombKey				bK;
	
	private BombLevel					bombLevelGame, bombLevelEditor;
	
	private JFrame						frame;
	
	@SuppressWarnings("unused")
	private boolean						bMenu, bEditor, bGame, bOptions;

	public BombMain()
	{
		super();
		
		this.fc.setCurrentDirectory(new File( System.getProperty("user.dir") ));
		this.fc.setFileFilter( this.cff );
		
		this.frame		= new JFrame();
		
		this.frame.setPreferredSize( new Dimension( 646, 512 ) );
		
		this.frame.setTitle( "=== BombSweety ===" );
		
		bK				= new BombKey( this );
		this.addKeyListener( bK );
		
		this.bombImage		= new BombImage( this );
		
		this.bombGame			= new BombGame( this, this.bombImage );
		this.bombMenu			= new BombMenu( this, this.bombImage );
		
		JPanel panel = (JPanel) this.frame.getContentPane();
		panel.setLayout(null);
		panel.setPreferredSize( new Dimension( 640,480 ) );
		panel.setLocation( 0, 0 );
		
		this.setBounds( 0, 0, 640, 480 );
		this.setSize( 640, 480 );
		
		panel.add( this );
        
		BombImage AFI	= new BombImage( this );
		this.frame.setIconImage( AFI.setPicsMain( "/images/Icon.gif", false ) );
		
		this.setIgnoreRepaint( true );
		
		this.frame.addWindowListener(new WindowAdapter()
			{
				public void windowClosing(WindowEvent e)
				{
					System.exit(0);
				}
			}
		);
		
		this.bombLevelGame			= new BombLevel( this.bombImage.setPicsMain( "/images/Wall.png", false ), this.bombImage.setPicsMain( "/images/Brick.png", false ), this.bombImage.setPicsMain( "/images/Brick_explode.png", false ), this.bombImage.setPicsMain( "/images/Sweety_Normal.png", false ), this.bombImage.setPicsMain( "/images/Sweety_Bomb.png", false ), this.bombImage.setPicsMain( "/images/Sweety_Tod.png", false ) );
		this.setLoadFirstLevel();
		
		this.bombLevelEditor		= new BombLevel( this.bombImage.setPicsMain( "/images/Wall.png", false ), this.bombImage.setPicsMain( "/images/Brick.png", false ), this.bombImage.setPicsMain( "/images/Brick_explode.png", false ), this.bombImage.setPicsMain( "/images/Sweety_Normal.png", false ), this.bombImage.setPicsMain( "/images/Sweety_Bomb.png", false ), this.bombImage.setPicsMain( "/images/Sweety_Tod.png", false ) );
		this.bombLevelEditor.setEmptyLevel(); 
		
		this.addMouseListener( this );
		this.addMouseMotionListener( this );
		
		this.requestFocus();
		
		this.bombEditor			= new BombEditor( this, this.bombImage );
		this.bombEditor.setBombLevelHud( this.bombLevelEditor );
		
		this.bGame		= false;
		this.bEditor	= false;
		this.bOptions	= false;
		this.bMenu		= true;
		
		this.frame.pack();
		this.frame.setResizable(false);
		this.frame.setVisible(true);
		
		this.createBufferStrategy(2);
		this.strategy		= this.getBufferStrategy();
		
		this.gameLoop();
	}

	public void setGame()
	{
		this.bEditor	= false;
		this.bMenu		= false;
		this.bOptions	= false;
		this.bGame		= true;
	}
	
	public void setEditor()
	{
		this.bEditor	= true;
		this.bMenu		= false;
		this.bOptions	= false;
		this.bGame		= false;
	}
	
	public void setMenu()
	{
		this.bEditor	= false;
		this.bMenu		= true;
		this.bOptions	= false;
		this.bGame		= false;
	}
	
	public Dimension getPreferredSize()
	{
		return new Dimension( 640, 480);
	}
	
	/**
	 * Set the title that should be displayed on the window
	 * 
	 * @param title The title to display on the window 
	 */
	public void setTitleInFrame(String title)
	{
		frame.setTitle(title);
	}
	
	public void gameLoop()
	{
		while ( this.bGameRunning )
		{
			//Get hold of a graphics context for the accelerated 
			// surface and blank it out
			this.g = (Graphics2D) strategy.getDrawGraphics();
			
			if ( ( this.bombGame != null ) && ( this.bGame ) )
			{
				this.bombGame.frameRenderer( g );
			} else if ( ( this.bombMenu != null ) && ( this.bMenu ) )
			{
				this.bombMenu.frameRenderer( g );
			} else if ( ( this.bombEditor != null ) && ( this.bEditor ) )
			{
				this.bombEditor.frameRenderer( g );
			}

			// finally, we've completed drawing so clear up the graphics
			// and flip the buffer over
			this.g.dispose();
			this.strategy.show();
		}
		System.exit( 0 );
	}
	
	public void save( Vector<BombWall> vWall, Vector<BombBrick> vBrick, Vector<BombSweety> vSweety, Vector<BombFinish> vFinish, int bombCount, String name, int level )
	{
		int p = this.fc.showSaveDialog(this);
		if(p == 0)
		{
			String s = this.fc.getSelectedFile().getPath();
			int t = s.indexOf(46);
			if ( t != -1 )
			{
				s	= s.substring( 0, t );
			}
			s	+= ".bomb";
			this.bombLevelEditor.setAll( vWall, vBrick, vSweety, vFinish, bombCount, name, level );
			this.bombLevelEditor.writeLevel( s );
		}
	}
	
	public void load( boolean bEditor )
	{
		int p = this.fc.showOpenDialog(this);
		if(p == 0)
		{
			String s = this.fc.getSelectedFile().getPath();
			if ( bEditor )
				this.bombLevelEditor.readLevel( s );
			else
				this.bombLevelGame.readLevel( s );
		}
	}
	
	public void setLoadFirstLevel()
	{
		String s	= System.getProperty("user.dir") + File.separator+"levels"+File.separator+ "Orginal.bomb";
		this.bombLevelGame.readLevel( s );
		this.bombGame.setLoadLevel( this.bombLevelGame, 0 );
	}
	
	public BombLevel getBombLevelGame()
	{
		return this.bombLevelGame;
	}
	
	public BombLevel getBombLevelEditor()
	{
		return this.bombLevelEditor;
	}
	
	public void setBombLevel( BombLevel bL )
	{
		this.bombLevelEditor	= bL;
	}
	
	public static void main(String[] args)
	{
		new BombMain();
	}

	public void mouseClicked(MouseEvent e)
	{
	}

	public void mousePressed(MouseEvent e)
	{
		int x = e.getX();       //ermitteln des x Wertes              
        int y = e.getY();       //ermitteln des y Wertes
        if ( ( this != null ) && ( e.getButton() == 1 ) && ( this.bGame ) )
        {
            this.bombGame.mousePressed( x, y );
        } else if ( ( this != null ) && ( e.getButton() == 1 ) && ( this.bMenu ) )
        {
            this.bombMenu.mousePressed( x, y );
        } else if ( ( this != null ) && ( e.getButton() == 1 ) && ( this.bEditor ) )
        {
            this.bombEditor.mousePressed( x, y );
        }
	}

	public void mouseReleased(MouseEvent e)
	{
		int x = e.getX();       //ermitteln des x Wertes              
        int y = e.getY();       //ermitteln des y Wertes
        if ( (this != null) && (e.getButton() == 1) && ( this.bGame ) )
        {
        	this.bombGame.mouseReleased( x, y );
        } else if ((this != null) && (e.getButton() == 3) && ( this.bGame ) )
        {
        	this.bombGame.mouseReleasedRight( x, y );
        } else if ( (this != null) && (e.getButton() == 1) && ( this.bMenu ) )
        {
        	this.bombMenu.mouseReleased( x, y );
        } else if ( (this != null) && (e.getButton() == 1) && ( this.bEditor ) )
        {
        	this.bombEditor.mouseReleased( x, y );
        } else if ((this != null) && (e.getButton() == 3) && ( this.bEditor ) )
        {
        	this.bombEditor.mouseReleasedRight( x, y );
        }
	}

	public void mouseEntered(MouseEvent e)
	{
	}

	public void mouseExited(MouseEvent e)
	{
	}

	public void mouseDragged(MouseEvent e)
	{
		int x = e.getX();
        int y = e.getY();
        if ( ( this != null ) && ( this.bGame ) )
        {
        	this.bombGame.mouseDragged( x, y );
        } else if ( ( this != null ) && ( this.bEditor ) )
        {
        	this.bombEditor.mouseDragged( x, y );
        }
	}

	public void mouseMoved(MouseEvent e)
	{
		int x = e.getX();
        int y = e.getY();
        if ( ( this != null ) && ( this.bGame ) )
        	this.bombGame.mouseMoved(x, y);
        else if ( ( this != null ) && ( this.bMenu ) )
        	this.bombMenu.mouseMoved(x, y);
        else if ( ( this != null ) && ( this.bEditor ) )
        	this.bombEditor.mouseMoved(x, y);
	}
	
	public void keyReleased( int key, char character )
	{
		if ( this.bGame )
			this.bombGame.keyReleased( key );
		else if ( this.bEditor )
			this.bombEditor.keyReleased( key, character );
	}
}
